/*
 * BreadCrumbNode.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Holds the url and the page title of a node.
    /// </summary>
    ///
    /// <remarks>
    /// This structure will be created by the <see cref="INodeDiscovery"/>
    /// (when discovering the current node) and by the <see cref="IPathDiscovery"/>
    /// implementations to return the path from the current node to the root node.
    /// Each node in that path will be respresented by this class.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    [Serializable]
    public struct BreadCrumbNode
    {
        /// <summary>
        /// Represents the title associated with this node.
        /// This variable is set by the constructor and is immutable.
        /// This value can be null if the title should default to the title found in the IPathDiscovery.
        /// This value should not be null if it's a node that is part of a path returned by the
        /// IPathDiscovery implementation.  This variable is referenced by the associated getter.
        /// </summary>
        private readonly string title;

        /// <summary>
        /// Represents the url (including sorted query strings if any) associated with this node.
        /// This variable is set by the constructor, is immutable and will never be null.
        /// This variable is referenced by the associated getter.
        /// </summary>
        private readonly string url;

        /// <summary>
        /// Gets the title associated with this node.
        /// </summary>
        ///
        /// <value>
        /// The title associated with this node, which could be null.
        /// It this value is null, the title from the site map should be used instead.
        /// </value>
        public string Title
        {
            get
            {
                return title;
            }
        }

        /// <summary>
        /// Gets the url (including sorted query strings if any) associated with this node.
        /// </summary>
        ///
        /// <value>
        /// The url (including sorted query strings if any) associated with this node.
        /// </value>
        public string Url
        {
            get
            {
                return url;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BreadCrumbNode"/> struct
        /// with the specified title and url.
        /// </summary>
        ///
        /// <param name="title">
        /// The title of the page (could be null or empty string).
        /// If an empty trimmed string is passed, null is stored instead.
        /// </param>
        /// <param name="url">
        /// The url of the page (including any sorted query strings).
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="url"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="url"/> is an empty trimmed string.
        /// </exception>
        public BreadCrumbNode(string title, string url)
        {
            if (url == null)
            {
                throw new ArgumentNullException("url", "'url' cannot be null");
            }
            if (url.Trim().Length == 0)
            {
                throw new ArgumentException("'url' cannot be an empty trimmed string", "url");
            }

            // store title as null if it's null or empty
            this.title = Helper.GetStringOrNull(title);
            this.url = url;
        }
    }
}
