/*
 * BreadCrumbTrailControl.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Util.ObjectFactory;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// The main class of the bread crumb trail component.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// This control will render the current node to the root node (or vise-versa) using a
    /// path separator and specific styles. The application can setup the separator and
    /// the various styles either by specifying default values in the configuration manager
    /// or by specifying attributes on the string. The object factory will be used to
    /// discover the node, the path to the root, and the formatting of the path
    /// using the key(s) specified on the control.
    /// </para>
    /// <para>
    /// This class will be directly embedded in an html page and used to render the bread crumb trail.
    /// This control supports IDE designer by exposing the various properties in attribute sheets.
    /// All attributes are also bindable to allow maximum flexibility. The user may override
    /// the path discovery process via the <see cref="DataSource"/> property to provide a custom path source
    /// (the path discovery process will be ignored then). The data source can be an <see cref="IList"/> or
    /// array of <see cref="BreadCrumbNode"/>(s), a <see cref="DataTable"/> with title/url columns, or
    /// a <see cref="DataSet"/> with at least one <see cref="DataTable"/> with title/url columns.
    /// </para>
    /// <para>
    /// Please install IIS and the Demo test files.
    /// Use your browser to open it. Here's the site map file for the demo:
    /// <code><![CDATA[
    /// <?xml version="1.0" encoding="UTF-8"?>
    ///
    /// <!-- a site-map for demo -->
    /// <BreadCrumb xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    ///             xsi:noNamespaceSchemaLocation="BreadCrumbTrailSchema.xsd"
    ///             version="v2">
    ///
    ///     <bc_node title="Main node" url="/BreadCrumbTrailWeb/Demo/Main.aspx" pattern="/Demo/Main.aspx$">
    ///         <bc_node title="Forums node" url="/BreadCrumbTrailWeb/Demo/Forums/Forums.aspx"
    ///                  pattern="/Demo/Forums/Forums.aspx$">
    ///             <bc_node title="Help node"
    ///                      url="/BreadCrumbTrailWeb/Demo/Forums/Help.aspx" pattern="/Demo/Forums/Help.aspx$"/>
    ///         </bc_node>
    ///     </bc_node>
    /// </BreadCrumb>
    /// ]]></code>
    /// Here's how to tag look like in aspx page:
    /// <code><![CDATA[
    /// <tc:BreadCrumbTrailControl
    ///    id="bcc"
    ///    Title="Main Page"
    ///    runat="server"/>
    /// <hr/>
    /// ]]></code>
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    [AspNetHostingPermission(SecurityAction.Demand, Level=AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level=AspNetHostingPermissionLevel.Minimal)]
    [DefaultProperty("PathSeparator")]
    [ToolboxData("<{0}:BreadCrumbTrailControl runat=\"server\"> </{0}:BreadCrumbTrailControl>")]
    [Designer(typeof(BreadCrumbTrailControlDesigner))]
    public class BreadCrumbTrailControl : System.Web.UI.WebControls.WebControl
    {
        /// <summary>
        /// The configuration namespace to load configuration values from.
        /// </summary>
        private static readonly string ConfigNamespace = typeof(BreadCrumbTrailControl).FullName;

        /// <summary>
        /// The configuration key for PathSeparator.
        /// </summary>
        private const string ConfigPathSeparator = "pathSeparator";

        /// <summary>
        /// The configuration key for PathSeparatorStyle.
        /// </summary>
        private const string ConfigPathSeparatorStyle = "pathSeparatorStyle";

        /// <summary>
        /// The configuration key for NodeStyle.
        /// </summary>
        private const string ConfigNodeStyle = "nodeStyle";

        /// <summary>
        /// The configuration key for RootNodeStyle.
        /// </summary>
        private const string ConfigRootNodeStyle = "rootNodeStyle";

        /// <summary>
        /// The configuration key for CurrentNodeStyle.
        /// </summary>
        private const string ConfigCurrentNodeStyle = "currentNodeStyle";

        /// <summary>
        /// The configuration key for MouseOverStyle.
        /// </summary>
        private const string ConfigMouseOverStyle = "mouseOverStyle";

        /// <summary>
        /// The configuration key for PathDirection.
        /// </summary>
        private const string ConfigPathDirection = "pathDirection";

        /// <summary>
        /// The configuration key for ErrorDisposition.
        /// </summary>
        private const string ConfigErrorDisposition = "errorDisposition";

        /// <summary>
        /// The configuration key for NodeDiscoveryKey.
        /// </summary>
        private const string ConfigNodeDiscoveryKey = "nodeDiscoveryKey";

        /// <summary>
        /// The configuration key for PathDiscoveryKey.
        /// </summary>
        private const string ConfigPathDiscoveryKey = "pathDiscoveryKey";

        /// <summary>
        /// The configuration key for PathWriterKey.
        /// </summary>
        private const string ConfigPathWriterKey = "pathWriterKey";

        /// <summary>
        /// Default NodeDiscoveryKey value used in RenderContents.
        /// </summary>
        private static readonly string DefaultNodeDiscoveryKey = typeof(INodeDiscovery).FullName;

        /// <summary>
        /// Default PathDiscoveryKey value used in RenderContents.
        /// </summary>
        private static readonly string DefaultPathDiscoveryKey = typeof(IPathDiscovery).FullName;

        /// <summary>
        /// Default PathWriterKey value used in RenderContents.
        /// </summary>
        private static readonly string DefaultPathWriterKey = typeof(IPathWriter).FullName;

        /// <summary>
        /// The column name for Title in the DataTable.
        /// </summary>
        private const string TitleColumn = "Title";

        /// <summary>
        /// The column name for Url in the DataTable.
        /// </summary>
        private const string UrlColumn = "Url";

        /// <summary>
        /// Represents the separator that will be drawn between nodes in a path.
        /// This variable is set initially to ":" and can be set in the constructor,
        /// is mutable and can be null to specify no path separator to use
        /// (empty string is allowed).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string pathSeparator = ":";

        /// <summary>
        /// Represents the separator CSS style class that will be used to draw the separator.
        /// It is assumed that the associated style class definitions (in the .css file)
        /// will be included on the resulting webpage. This variable is set initially to null
        /// and can be set in the constructor, is mutable and can be null to specify
        /// no path separator style to use (empty [trim'd] strings should be translated into a null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string pathSeparatorStyle;

        /// <summary>
        /// Represents the non-root, non-current node CSS style class that will be used to draw the node.
        /// It is assumed that the associated style class definitions (in the .css file)
        /// will be included on the resulting webpage. This variable is set initially to null
        /// and can be set in the constructor, is mutable and can be null to specify
        /// no node style to use (empty [trim'd] strings should be translated into a null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string nodeStyle;

        /// <summary>
        /// Represents the current (only) node CSS style class that will be used to draw the current node.
        /// It is assumed that the associated style class definitions (in the .css file)
        /// will be included on the resulting webpage. This variable is set initially to null
        /// and can be set in the constructor, is mutable and can be null to specify
        /// no current node style to use (empty [trim'd] strings should be translated into a null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string currentNodeStyle;

        /// <summary>
        /// Represents the root (only) node CSS style class that will be used to draw the root node.
        /// It is assumed that the associated style class definitions (in the .css file)
        /// will be included on the resulting webpage. This variable is set initially to null
        /// and can be set in the constructor, is mutable and can be null to specify
        /// no root node style to use (empty [trim'd] strings should be translated into a null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string rootNodeStyle;

        /// <summary>
        /// Represents the CSS style class that will be used to change the nodes style
        /// when the mouse is over it. It is assumed that the associated style class definitions
        /// (in the .css file) will be included on the resulting webpage.
        /// This variable is set initially to null and can be set in the constructor,
        /// is mutable and can be null to specify no mouse over style to use
        /// (empty [trim'd] strings should be translated into a null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string mouseOverStyle;

        /// <summary>
        /// Represents whether any exceptions should be silently ignore, errors shown,
        /// or exceptions to be thrown.
        /// This variable is set by the constructor, is mutable and can only be a valid enum value.
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private ErrorDisposition errorDisposition = ErrorDisposition.SilentlyIgnore;

        /// <summary>
        /// Represents the direction of the path that will be rendered.
        /// This variable is set by the constructor, is mutable and can only be a valid enum value.
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private PathDirection pathDirection = PathDirection.RootToCurrent;

        /// <summary>
        /// Represents the data source to use (overriding the path discovery process) in determining the current path.
        /// The object passed can be an:
        /// <ol>
        /// <li>IList of BreadCrumbNode elements.</li>
        /// <li>Array of BreadCrumbNode elements.</li>
        /// <li>A DataTable that has columns of "Title" and "Url" defined in it.</li>
        /// <li>A DataSet containing at least one DataTable with columns defined above.</li>
        /// </ol>
        /// This variable is initially null, is mutable and can be null to specify no path override.
        /// This variable will be referenced by the associated property and in the DataBind method.
        /// </summary>
        private object dataSource;

        /// <summary>
        /// Represents the title that is assigned to the page this control appears in.
        /// If no title is specified (null), the title found in the static site map
        /// will be used instead (if none is found, "Unknown" will be rendered instead).
        /// This variable is initially null, is mutable and can be null to specify no specific
        /// title to use (empty [trim'd] strings should be translated into a null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string title;

        /// <summary>
        /// Represents the Object Factory key that will be used when the
        /// INodeDiscovery implementation is created. If no key is specified (ie. it's null),
        /// the INodeDiscovery type's full name will be used instead.
        /// This variable is initially null and can be set in the constructor, is mutable and
        /// can be null (any empty [trim'd] string should be translated to null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string nodeDiscoveryKey;

        /// <summary>
        /// Represents the Object Factory key that will be used when the
        /// IPathDiscovery implementation is created. If no key is specified (ie. it's null),
        /// the IPathDiscovery type's full name will be used instead.
        /// This variable is initially null and can be set in the constructor, is mutable and
        /// can be null (any empty [trim'd] string should be translated to null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string pathDiscoveryKey;

        /// <summary>
        /// Represents the Object Factory key that will be used when the
        /// IPathWriter implementation is created. If no key is specified (ie. it's null),
        /// the IPathWriter type's full name will be used instead.
        /// This variable is initially null and can be set in the constructor, is mutable and
        /// can be null (any empty [trim'd] string should be translated to null).
        /// This variable will be referenced by the associated property and in the Render method.
        /// </summary>
        private string pathWriterKey;

        /// <summary>
        /// Represents the path override that should be used (in place of the path discovery process).
        /// This variable is initially null, is mutable and can be null (but not an empty size)
        /// if no path override is in place. This variable is only set to non-null in the
        /// DataBind method if a dataSource has been specified.
        /// The variable is then referenced in the Render method to determine whether
        /// the path discovery process should be used (if this variable is null) or
        /// not (if this variable is non-null).
        /// </summary>
        private BreadCrumbNode[] pathOverride;

        /// <summary>
        /// Gets or sets the path separator that is drawn between nodes in the path.
        /// </summary>
        ///
        /// <value>
        /// The path separator string (could be null to indicate no path separator).
        /// An empty trimmed string is allowed.
        /// </value>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue(":")]
        [Description("The path separator to use.")]
        public string PathSeparator
        {
            get
            {
                return pathSeparator;
            }
            set
            {
                pathSeparator = value;
            }
        }

        /// <summary>
        /// Gets or sets the path separator CSS style class that will be used to draw the separator.
        /// </summary>
        ///
        /// <value>
        /// The path separator style string (could be null to indicate no path separator style).
        /// If setting to an empty trimmed string, it'll be treated like null.
        /// </value>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("The CSS Style Class to use for the path separator.")]
        public string PathSeparatorStyle
        {
            get
            {
                return pathSeparatorStyle;
            }
            set
            {
                pathSeparatorStyle = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the non-root, non-current node CSS style class that will be used to draw the node.
        /// </summary>
        ///
        /// <value>
        /// The non-root, non-current node CSS style string (could be null to indicate no style).
        /// If setting to an empty trimmed string, it'll be treated like null.
        /// </value>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("The CSS Style Class to use for each node in the path.")]
        public string NodeStyle
        {
            get
            {
                return nodeStyle;
            }
            set
            {
                nodeStyle = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the current node CSS style class that will be used to draw the current node.
        /// </summary>
        ///
        /// <value>
        /// The current node CSS style string (could be null to indicate no style).
        /// If setting to an empty trimmed string, it'll be treated like null.
        /// </value>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("The CSS Style Class to use for the current node in the path.")]
        public string CurrentNodeStyle
        {
            get
            {
                return currentNodeStyle;
            }
            set
            {
                currentNodeStyle = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the root node CSS style class that will be used to draw the root node.
        /// </summary>
        ///
        /// <value>
        /// The root node CSS style string (could be null to indicate no style).
        /// If setting to an empty trimmed string, it'll be treated like null.
        /// </value>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("The CSS Style Class to use for the root node in the path.")]
        public string RootNodeStyle
        {
            get
            {
                return rootNodeStyle;
            }
            set
            {
                rootNodeStyle = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the mouse over CSS style class that will be used to change the node style
        /// when the mouse is over it.
        /// </summary>
        ///
        /// <value>
        /// The mouse over CSS style string (could be null to indicate no style).
        /// If setting to an empty trimmed string, it'll be treated like null.
        /// </value>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("The CSS Style Class to use when the mouse hovers over a node in the path.")]
        public string MouseOverStyle
        {
            get
            {
                return mouseOverStyle;
            }
            set
            {
                mouseOverStyle = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the error disposition method, whether to silently ignore, render exception,
        /// or rethrow to IIS engine.
        /// </summary>
        ///
        /// <value>
        /// The error disposition method.
        /// </value>
        ///
        /// <exception cref="ArgumentException">
        /// Setting to an invalid enumeration value.
        /// </exception>
        [Bindable(true)]
        [Category("Behavior")]
        [DefaultValue(ErrorDisposition.SilentlyIgnore)]
        [Description("The disposition of any errors encountered during rendering.")]
        public ErrorDisposition ErrorDisposition
        {
            get
            {
                return errorDisposition;
            }
            set
            {
                if (!Enum.IsDefined(typeof(ErrorDisposition), value))
                {
                    throw new ArgumentException("ErrorDisposition value is undefined", "ErrorDisposition");
                }
                errorDisposition = value;
            }
        }

        /// <summary>
        /// Gets or sets the direction of the path that will be rendered,
        /// whether root to current node or current to root node.
        /// </summary>
        ///
        /// <value>
        /// The direction of the path.
        /// </value>
        ///
        /// <exception cref="ArgumentException">
        /// Setting to an invalid enumeration value.
        /// </exception>
        [Bindable(true)]
        [Category("Behavior")]
        [DefaultValue(PathDirection.RootToCurrent)]
        [Description("The direction the path should be rendered.")]
        public PathDirection PathDirection
        {
            get
            {
                return pathDirection;
            }
            set
            {
                if (!Enum.IsDefined(typeof(PathDirection), value))
                {
                    throw new ArgumentException("PathDirection value is undefined", "PathDirection");
                }
                pathDirection = value;
            }
        }

        /// <summary>
        /// Gets or sets the data source representing the path to use.
        /// Setting this will override the path discovery process.
        /// </summary>
        ///
        /// <value>
        /// The data source representing the path to use (could be null).
        /// See exception for possible data types.
        /// </value>
        ///
        /// <exception cref="ArgumentException">
        /// Setting to an <see cref="IList"/> that is empty, contains null,
        /// or non <see cref="BreadCrumbNode"/> element
        /// -or-
        /// setting to a BreadCrumbNode[] that is empty or contains null url (uninitialized)
        /// -or-
        /// setting to a DataTable and does not contain a Title and Url column
        /// -or-
        /// setting to a DataSet that has no tables or it's first table fails the DataTable check above.
        /// -or-
        /// setting to anything non-null other than all above.
        /// </exception>
        [Bindable(true)]
        [Category("Data")]
        [DefaultValue(null)]
        [Description("The override for the nodes in the path.")]
        public object DataSource
        {
            get
            {
                return dataSource;
            }
            set
            {
                // check supported type
                CheckDataSource(value);
                dataSource = value;
            }
        }

        /// <summary>
        /// Gets or sets the title that is assigned to the page this control appears in.
        /// </summary>
        ///
        /// <remarks>
        /// If no title is specified (null), the title found in the static web site is used instead.
        /// If none is found, "Unknown" will be rendered.
        /// </remarks>
        ///
        /// <value>
        /// The title of the page (could be null).
        /// If setting to an empty trimmed string, it'll be treated like null.
        /// </value>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("The title override to use for the current page.")]
        public string Title
        {
            get
            {
                return title;
            }
            set
            {
                title = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the object factory key of the <see cref="INodeDiscovery"/> implementation to use.
        /// If not specified <see cref="INodeDiscovery"/>'s full name is used.
        /// </summary>
        ///
        /// <value>
        /// The object factory key of the <see cref="INodeDiscovery"/> implementation to use
        /// (could be null).
        /// </value>
        [Bindable(true)]
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("The Object Factory key to use in creating the INodeDiscovery implementation.")]
        public string NodeDiscoveryKey
        {
            get
            {
                return nodeDiscoveryKey;
            }
            set
            {
                nodeDiscoveryKey = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the object factory key of the <see cref="IPathDiscovery"/> implementation to use.
        /// If not specified <see cref="IPathDiscovery"/>'s full name is used.
        /// </summary>
        ///
        /// <value>
        /// The object factory key of the <see cref="IPathDiscovery"/> implementation to use
        /// (could be null).
        /// </value>
        [Bindable(true)]
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("The Object Factory key to use in creating the IPathDiscovery implementation.")]
        public string PathDiscoveryKey
        {
            get
            {
                return pathDiscoveryKey;
            }
            set
            {
                pathDiscoveryKey = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Gets or sets the object factory key of the <see cref="IPathWriter"/> implementation to use.
        /// If not specified <see cref="IPathWriter"/>'s full name is used.
        /// </summary>
        ///
        /// <value>
        /// The object factory key of the <see cref="IPathWriter"/> implementation to use
        /// (could be null).
        /// </value>
        [Bindable(true)]
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("The Object Factory key to use in creating the IPathWriter implementation.")]
        public string PathWriterKey
        {
            get
            {
                return pathWriterKey;
            }
            set
            {
                pathWriterKey = Helper.GetStringOrNull(value);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BreadCrumbTrailControl"/> class
        /// using configuration manager to set default values.
        /// </summary>
        ///
        /// <remarks>
        /// Configuration will be retrieved from the namespace:
        /// <c>TopCoder.Web.UI.WebControl.BreadCrumb.BreadCrumbTrailControl</c>.
        /// </remarks>
        public BreadCrumbTrailControl()
        {
            // get default configuration values, no exception will be thrown
            PathSeparator = GetString(ConfigNamespace, ConfigPathSeparator, PathSeparator);
            PathSeparatorStyle = GetString(ConfigNamespace, ConfigPathSeparatorStyle, PathSeparatorStyle);
            NodeStyle = GetString(ConfigNamespace, ConfigNodeStyle, NodeStyle);
            CurrentNodeStyle = GetString(ConfigNamespace, ConfigCurrentNodeStyle, CurrentNodeStyle);
            RootNodeStyle = GetString(ConfigNamespace, ConfigRootNodeStyle, RootNodeStyle);
            MouseOverStyle = GetString(ConfigNamespace, ConfigMouseOverStyle, MouseOverStyle);
            ErrorDisposition = (ErrorDisposition)
                GetEnum(ConfigNamespace, ConfigErrorDisposition, ErrorDisposition, ErrorDisposition.GetType());
            PathDirection = (PathDirection)
                GetEnum(ConfigNamespace, ConfigPathDirection, PathDirection, PathDirection.GetType());
            NodeDiscoveryKey = GetString(ConfigNamespace, ConfigNodeDiscoveryKey, NodeDiscoveryKey);
            PathDiscoveryKey = GetString(ConfigNamespace, ConfigPathDiscoveryKey, PathDiscoveryKey);
            PathWriterKey = GetString(ConfigNamespace, ConfigPathWriterKey, PathWriterKey);
        }

        /// <summary>
        /// Determines and renders the current path to the specified writer.
        /// </summary>
        ///
        /// <remarks>
        /// This will first discover current node using INodeDiscovery,
        /// then discovery the current path using IPathDiscovery,
        /// and finally write the path using IPathWriter.
        /// Any exception is rethrown, ignored, or rendered depending on the ErrorDisposition value.
        /// </remarks>
        ///
        /// <param name="writer">
        /// The writer to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Writer is a null reference.
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// Error determining/rendering path and ErrorDisposition is ThrowException.
        /// </exception>
        protected override void RenderContents(HtmlTextWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer", "'writer' cannot be null");
            }

            // accumulate exception
            ArrayList ex = new ArrayList();
            try
            {
                // create object factory with this namespace
                ObjectFactory factory = ObjectFactory.GetDefaultObjectFactory(ConfigNamespace);

                BreadCrumbNode[] currentPath = null;
                if (pathOverride == null)
                {
                    // 1. Discover current node (ISiteDiscovery)
                    string key = ((NodeDiscoveryKey == null) ? DefaultNodeDiscoveryKey : NodeDiscoveryKey);
                    INodeDiscovery nodeDiscovery = (INodeDiscovery) Helper.CreateInstance(
                        factory, null, key, typeof(INodeDiscovery));
                    BreadCrumbNode currentNode = nodeDiscovery.GetCurrentNode(Title, Page);

                    // 2. Discover current path (IPathDiscovery)
                    key = ((PathDiscoveryKey == null) ? DefaultPathDiscoveryKey : PathDiscoveryKey);
                    IPathDiscovery pathDiscovery = (IPathDiscovery) Helper.CreateInstance(
                        factory, null, key, typeof(IPathDiscovery));
                    currentPath = pathDiscovery.GetPath(currentNode, Page);
                }
                else
                {
                    // use pathOverride
                    currentPath = pathOverride;
                }

                // 3. Write the path (IPathWriter)
                if ((currentPath == null) || (currentPath.Length == 0))
                {
                    // no path to render
                    return;
                }

                string pwKey = ((PathWriterKey == null) ? DefaultPathWriterKey : PathWriterKey);
                IPathWriter pathWriter = (IPathWriter) Helper.CreateInstance(
                    factory, null, pwKey, typeof(IPathWriter));
                BreadCrumbException[] writeExceptions = pathWriter.WritePath(writer, currentPath, this);
                if (writeExceptions != null)
                {
                    // accumulate exception
                    ex.AddRange(writeExceptions);
                }
            }
            catch (Exception e)
            {
                // accumulate in list and encapsulate in BreadCrumbException
                // if ErrorDisposition is ThrowException, it is rethrown
                Helper.HandleException(e, "Error rendering bread crumb trail", ex, ErrorDisposition, true);
            }

            // 4. Render errors (if any and requested)
            if ((ex.Count > 0) && (ErrorDisposition == ErrorDisposition.RenderException))
            {
                RenderErrors(writer, ex);
            }
        }

        /// <summary>
        /// Binds a data source to the invoked server control and all its child controls.
        /// </summary>
        ///
        /// <exception cref="ArgumentException">
        /// DataSource is an <see cref="IList"/> that is empty, contains null,
        /// or non <see cref="BreadCrumbNode"/> element
        /// -or-
        /// is a BreadCrumbNode[] that is empty or contains null url (uninitialized)
        /// -or-
        /// is a DataTable and does not contain a Title and Url column
        /// -or-
        /// is a DataSet that has no tables or it's first table fails the DataTable check above.
        /// -or-
        /// is anything non-null other than all above.
        /// </exception>
        public override void DataBind()
        {
            base.DataBind();

            // set pathOverride depending on the DataSource type
            pathOverride = null;
            object ds = DataSource; // cache getter result so we don't invoke too many method call

            // no DataSource, done
            if (ds == null)
            {
                return;
            }

            // BreadCrumbNode[], copy it to pathOverride
            if (ds is BreadCrumbNode[])
            {
                pathOverride = (BreadCrumbNode[]) ds;
                return;
            }

            // IList of BreadCrumbNode, convert to array
            if (ds is IList)
            {
                pathOverride = (BreadCrumbNode[])
                    new ArrayList((IList) ds).ToArray(typeof(BreadCrumbNode));
                return;
            }

            // DataTable or DataSet, convert to IList and return as array
            if ((ds is DataTable) || (ds is DataSet))
            {
                DataTable t = ((ds is DataTable) ? (DataTable) ds : (((DataSet) ds).Tables[0]));
                // only if there's a row
                if (t.Rows.Count > 0)
                {
                    ArrayList rowList = new ArrayList();

                    // iterate row and append BreadCrumbNode to arraylist
                    foreach (DataRow row in t.Rows)
                    {
                        rowList.Add(new BreadCrumbNode(row[TitleColumn].ToString(), row[UrlColumn].ToString()));
                    }

                    // convert to array and done
                    pathOverride = (BreadCrumbNode[]) rowList.ToArray(typeof(BreadCrumbNode));
                }
            }
        }

        /// <summary>
        /// Checks that the data source is of supported type.
        /// </summary>
        ///
        /// <param name="data">
        /// The data source to check.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// <paramref name="data"/> is an <see cref="IList"/> that is empty, contains null,
        /// or non <see cref="BreadCrumbNode"/> element
        /// -or-
        /// is a BreadCrumbNode[] that is empty or contains null url (uninitialized)
        /// -or-
        /// is a DataTable and does not contain a Title and Url column
        /// -or-
        /// is a DataSet that has no tables or it's first table fails the DataTable check above.
        /// -or-
        /// is anything non-null other than all above.
        /// </exception>
        private void CheckDataSource(object data)
        {
            if (data is BreadCrumbNode[])
            {
                CheckBcArray((BreadCrumbNode[]) data);
            }
            else if (data is IList)
            {
                CheckBcIList((IList) data);
            }
            else if (data is DataTable)
            {
                CheckBcDataTable((DataTable) data);
            }
            else if (data is DataSet)
            {
                CheckBcDataSet((DataSet) data);
            }
            else if (data != null)
            {
                throw new ArgumentException("Unsupported datatype", "DataSource");
            }
        }

        /// <summary>
        /// Checks the array is not empty and contains no uninitialized element (Url is null).
        /// </summary>
        ///
        /// <param name="dataSource">
        /// The array to check.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// Any check fails.
        /// </exception>
        private void CheckBcArray(BreadCrumbNode[] dataSource)
        {
            if (dataSource.Length == 0)
            {
                throw new ArgumentException("'DataSource' cannot be an empty array", "DataSource");
            }

            // all element must be initialized (no null Url)
            foreach (BreadCrumbNode node in dataSource)
            {
                if (node.Url == null)
                {
                    throw new ArgumentException(
                        "'DataSource' cannot contain uninitialized element with null Url", "DataSource");
                }
            }
        }

        /// <summary>
        /// Checks the IList is not empty, contains non BreadCrumbNode (or null), and
        /// contains uninitialized element (Url is null).
        /// </summary>
        ///
        /// <param name="dataSource">
        /// The IList to check.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// Any check fails.
        /// </exception>
        private void CheckBcIList(IList dataSource)
        {
            if (dataSource.Count == 0)
            {
                throw new ArgumentException("'DataSource' cannot be an empty list", "DataSource");
            }

            // all element must be BreadCrumbNode and initialized (no null Url)
            foreach (object node in dataSource)
            {
                if (!(node is BreadCrumbNode))
                {
                    throw new ArgumentException(
                        "'DataSource' cannot contains null nor non-BreadCrumbNode element", "DataSource");
                }

                if (((BreadCrumbNode) node).Url == null)
                {
                    throw new ArgumentException(
                        "'DataSource' cannot contain uninitialized element with null Url", "DataSource");
                }
            }
        }

        /// <summary>
        /// Checks the DataTable has a Title and Url columns defined.
        /// </summary>
        ///
        /// <param name="dataSource">
        /// The DataTable to check.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// Any check fails.
        /// </exception>
        private void CheckBcDataTable(DataTable dataSource)
        {
            if (dataSource.Columns.IndexOf(TitleColumn) == -1)
            {
                throw new ArgumentException("DataTable must have a Title column", "DataSource");
            }

            if (dataSource.Columns.IndexOf(UrlColumn) == -1)
            {
                throw new ArgumentException("DataTable must have a Url column", "DataSource");
            }
        }

        /// <summary>
        /// Checks the DataSet is not empty and the first table has a Title and Url columns defined.
        /// </summary>
        ///
        /// <param name="dataSource">
        /// The DataSet to check.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// Any check fails.
        /// </exception>
        private void CheckBcDataSet(DataSet dataSource)
        {
            if (dataSource.Tables.Count == 0)
            {
                throw new ArgumentException("DataSet must contain at least one table", "DataSource");
            }

            // check table has Url and Title
            CheckBcDataTable(dataSource.Tables[0]);
        }

        /// <summary>
        /// Gets a string value from the configuration namespace and key.
        /// If null or empty trimmed string, the default value is returned.
        /// Any error will be ignored.
        /// </summary>
        ///
        /// <param name="nameSpace">
        /// The namespace of the configuration.
        /// </param>
        /// <param name="key">
        /// The property key.
        /// </param>
        /// <param name="defaultValue">
        /// The default value to return if the configuration doesn't exist or is empty.
        /// </param>
        ///
        /// <returns>
        /// The configuration value or the default value string.
        /// </returns>
        private string GetString(string nameSpace, string key, string defaultValue)
        {
            try
            {
                ConfigManager cm = ConfigManager.GetInstance();
                string val = cm.GetValue(nameSpace, key);

                // return default if null/empty, otherwise return the config value
                return (((val == null) || (val.Trim().Length == 0)) ? defaultValue : val);
            }
            catch (Exception)
            {
                // ignore and return defaultValue
                return defaultValue;
            }
        }

        /// <summary>
        /// Gets a string/integer value from the configuration namespace and key
        /// and return the enumeration value.
        /// If null or empty trimmed string, the default value is returned.
        /// Any error will be ignored.
        /// </summary>
        ///
        /// <param name="nameSpace">
        /// The namespace of the configuration.
        /// </param>
        /// <param name="key">
        /// The property key.
        /// </param>
        /// <param name="defaultValue">
        /// The default value to return if the configuration doesn't exist or is empty
        /// or is an invalid enum value.
        /// </param>
        /// <param name="type">
        /// The enumeration type.
        /// </param>
        ///
        /// <returns>
        /// The configuration value or the default value.
        /// </returns>
        private object GetEnum(string nameSpace, string key, object defaultValue, Type type)
        {
            try
            {
                string val = GetString(nameSpace, key, null);

                // return default if not defined or empty
                if ((val == null) || (val.Trim().Length == 0))
                {
                    return defaultValue;
                }

                // parse it and check it's defined (return default if undefined)
                object enumVal = Enum.Parse(type, val);
                return (Enum.IsDefined(type, enumVal) ? enumVal : defaultValue);
            }
            catch (Exception)
            {
                // ignore and return defaultValue
                return defaultValue;
            }
        }

        /// <summary>
        /// Renders errors to the html writer.
        /// This method is called by RenderContents when ErrorDisposition is RenderException.
        /// </summary>
        ///
        /// <param name="writer">
        /// The writer to use.
        /// </param>
        /// <param name="ex">
        /// The exception list to render.
        /// </param>
        private void RenderErrors(HtmlTextWriter writer, IList ex)
        {
            try
            {
                writer.RenderBeginTag(HtmlTextWriterTag.P);
                foreach (BreadCrumbException e in ex)
                {
                    writer.Write(HttpUtility.HtmlEncode(e.Message));
                }
                writer.RenderEndTag();
            }
            catch (Exception)
            {
            }
        }
    }
}
