/*
 * Helper.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.Collections;
using TopCoder.Util.ObjectFactory;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Several static helper functions for handling exceptions.
    /// </summary>
    internal sealed class Helper
    {
        /// <summary>
        /// Prevents this class from being instantiated.
        /// </summary>
        private Helper()
        {
        }

        /// <summary>
        /// Returns the string or null if the given string is empty.
        /// </summary>
        ///
        /// <param name="str">
        /// The string to check (could be null).
        /// </param>
        ///
        /// <returns>
        /// null if <paramref name="str"/> is null or an empty trimmed string;
        /// otherwise <paramref name="str"/> is returned.
        /// </returns>
        internal static string GetStringOrNull(string str)
        {
            return (((str == null) || (str.Trim().Length == 0)) ? null : str);
        }

        /// <summary>
        /// Handles exception depending on the error disposition given.
        /// </summary>
        ///
        /// <remarks>
        /// If error disposition is SilentlyIgnore, the exception is ignored.
        /// If error disposition is RenderException, the exception will be encapsulated
        /// into BreadCrumbException (if it's not a BreadCrumbException) and added to the array list.
        /// If error disposition is ThrowException, similar behavior to RenderException and
        /// then it's re-thrown (if requested).
        /// </remarks>
        ///
        /// <param name="e">
        /// The exception to handle.
        /// </param>
        /// <param name="message">
        /// The error message to encapsulate the exception if it's not a BreadCrumbException.
        /// </param>
        /// <param name="exList">
        /// The list to accumulate exceptions to.
        /// </param>
        /// <param name="disposition">
        /// The error disposition which determines what to do with the exception.
        /// </param>
        /// <param name="rethrow">
        /// Whether to rethrow exception if disposition is ThrowException.
        /// </param>
        ///
        /// <exception cref="BreadCrumbException">
        /// If disposition is ThrowException and <paramref name="rethrow"/> is true.
        /// </exception>
        internal static void HandleException(
            Exception e, string message, IList exList, ErrorDisposition disposition, bool rethrow)
        {
            if (disposition == ErrorDisposition.SilentlyIgnore)
            {
                // ignore
                return;
            }

            // encapsulate in BreadCrumbException if necessary
            if (!(e is BreadCrumbException))
            {
                e = new BreadCrumbException(message, e);
            }

            // add to list and re-throw if ThrowException
            exList.Add(e);
            if ((disposition == ErrorDisposition.ThrowException) && (rethrow))
            {
                throw e;
            }
        }

        /// <summary>
        /// Creates a class instance using object factory, passing the namespace and key.
        /// Additionally this will perform a type check to make sure we have the expected type.
        /// </summary>
        ///
        /// <param name="factory">
        /// The object factory to use. This could be null which means
        /// a default object factory will be instantiated with default namespace.
        /// </param>
        /// <param name="nameSpace">
        /// The namespace to get the object factory with.
        /// This could be null if an object factory is given.
        /// </param>
        /// <param name="key">
        /// The key containing the class definition.
        /// </param>
        /// <param name="expected">
        /// The expected type of the class.
        /// </param>
        ///
        /// <returns>
        /// The object instance.
        /// </returns>
        ///
        /// <exception cref="BreadCrumbException">
        /// If fail instantiating or not of the expected type.
        /// </exception>
        internal static object CreateInstance(
            ObjectFactory factory, string nameSpace, string key, Type expected)
        {
            try
            {
                if (factory == null)
                {
                    // instantiate object factory with the namespace
                    factory = ObjectFactory.GetDefaultObjectFactory(nameSpace);
                }

                // create defined object in the key
                object instance = factory.CreateDefinedObject(key);

                // check type
                if (!(expected.IsAssignableFrom(instance.GetType())))
                {
                    throw new BreadCrumbException(string.Format(
                        "Fail instantiating: namespace {0} key {1} should contain a type of {2}",
                        nameSpace, key, expected.FullName));
                }
                return instance;
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException(string.Format(
                    "Fail instantiating {0} from namespace {1} key {2}",
                    expected.FullName, nameSpace, key), e);
            }
        }
    }
}
