/*
 * HttpRequestNodeDiscovery.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Collections.Specialized;
using System.Text;
using System.Web;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.NodeDiscovery
{
    /// <summary>
    /// Discovers the current node from the <see cref="HttpRequest"/> for the given page.
    /// </summary>
    ///
    /// <remarks>
    /// This implementation will return a valid Url (with query strings if specified) in the
    /// <see cref="GetCurrentNode"/> method. The query string will be appended to the Url in a sorted order.
    /// Note that the url will be in decoded format.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class HttpRequestNodeDiscovery : INodeDiscovery
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HttpRequestNodeDiscovery"/> class.
        /// </summary>
        public HttpRequestNodeDiscovery()
        {
            // do nothing
        }

        /// <summary>
        /// Discovers the node url from an <see cref="HttpRequest"/> on the page and
        /// creates a <see cref="BreadCrumbNode"/> from the resulting title, url with query strings sorted.
        /// </summary>
        ///
        ///
        /// <param name="title">
        /// A possibly null or empty trimmed title to use.
        /// </param>
        /// <param name="page">
        /// A non-null page to retrieve the node for.
        /// </param>
        ///
        /// <returns>
        /// A <see cref="BreadCrumbNode"/> representation of the current url.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="page"/> is a null reference.
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// Any error occurs determining the current node.
        /// </exception>
        public BreadCrumbNode GetCurrentNode(string title, Page page)
        {
            if (page == null)
            {
                throw new ArgumentNullException("page", "'page' cannot be null");
            }

            try
            {
                // get the path (without hostname) and query string from the page's HttpRequest
                HttpRequest req = page.Request;
                string path = req.Path;
                NameValueCollection queryColl = req.QueryString;

                // rebuild the url: path + [query-string (sorted by param name)]
                //                  where query-string is ?param1=val1&param2=val2&...
                StringBuilder url = new StringBuilder();
                url.Append(path);

                bool first = true;
                string[] keys = queryColl.AllKeys;
                Array.Sort(keys);
                foreach (string key in keys)
                {
                    foreach (string val in queryColl.GetValues(key))
                    {
                        // prefix with '?' for 1st param, or with '&' for subsequent params
                        url.Append((first) ? '?' : '&');
                        first = false;
                        url.AppendFormat("{0}={1}", key, val);
                    }
                }

                return new BreadCrumbNode(title, url.ToString());
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error determining the current node", e);
            }
        }
    }
}
