/*
 * DynamicPathDiscovery.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// An implementation of <see cref="IPathDiscovery"/> that attempts to create a static site map
    /// and then apply dynamic information to that site map.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// The dynamic information can either be in matching nodes with different query strings
    /// or in new nodes that will be assumed to map to the last known node.
    /// </para>
    /// <para>
    /// This implementation will use an <see cref="ISiteDiscovery"/> to discover the site map.
    /// That site map will then be used to help determine where the current node belongs and
    /// what it's path back to the root node is.
    /// This class will be used by the <see cref="BreadCrumbTrailControl"/> to discover the path
    /// from the given node to the root node. This class will use session level attributes
    /// to store the current path that is known and the current sitemap [ie. the state information].
    /// A reverse site map is also internally managed to enable faster path finding to the root.
    /// This class will only be created by the object manager when a path discovery object is needed
    /// by the <see cref="BreadCrumbTrailControl"/>.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class DynamicPathDiscovery : IPathDiscovery
    {
        /// <summary>
        /// The object to be passed to PathAlg.FindShortestToRoot.
        /// This contains several data structures related to finding the best shortest path to root.
        /// See algorithm in OnRootFound().
        /// </summary>
        private sealed class ShortestPathResult
        {
            /// <summary>
            /// The current node to get the path for.
            /// </summary>
            public BreadCrumbNode CurrentNode;

            /// <summary>
            /// The current path.
            /// </summary>
            public NodePair[] CurrentPath = null;

            /// <summary>
            /// The best path found so far. IList of NodePair.
            /// </summary>
            public ArrayList BestPath = null;

            /// <summary>
            /// The best commonality count of BestPath with CurrentPath.
            /// CurrentNode is included in this count.
            /// </summary>
            public int BestCommonCount = -1;
        }

        /// <summary>
        /// Defines the session scope attribute name that will contain the current known path.
        /// </summary>
        ///
        /// <remarks>
        /// This constant will be used by this class to retrieve the current path.
        /// The current path will be of NodePair[] type representing the known path
        /// from the root (0th index) to the prior page ((length-1)th index).
        /// </remarks>
        public static readonly string CURRENT_PATH = typeof(DynamicPathDiscovery).FullName + ".currentPath";

        /// <summary>
        /// Defines the session scope attribute name that will contain the site map.
        /// </summary>
        ///
        /// <remarks>
        /// This constant will be used by this class to retrieve the site map.
        /// The site map will be an <see cref="IDictionary"/> with <see cref="INodeMatcher"/>
        /// as the key and an <see cref="IList"/> of <see cref="INodeMatcher"/> as the value and can be empty.
        /// This <see cref="IDictionary"/> map represents the <see cref="INodeMatcher"/> parent to its children
        /// <see cref="INodeMatcher"/>(s) if any. The list can be empty if there are no children of the parent
        /// (ie. a leaf node).
        /// </remarks>
        public static readonly string CURRENT_SITEMAP = typeof(DynamicPathDiscovery).FullName + ".siteMap";

        /// <summary>
        /// Defines the Session scope attribute name that will contain the reverse site map mapping.
        /// </summary>
        ///
        /// <remarks>
        /// Similar to <see cref="CURRENT_SITEMAP"/>, however the IDictionary will contain
        /// a mapping from children to multiple parent.
        /// The root node will have an empty value list.
        /// </remarks>
        public static readonly string CURRENT_SITEMAP_REVERSE = CURRENT_SITEMAP + ".reverse";

        /// <summary>
        /// Initializes a new instance of the <see cref="DynamicPathDiscovery"/> class.
        /// It does nothing at the moment.
        /// </summary>
        public DynamicPathDiscovery()
        {
            // do nothing
        }

        /// <summary>
        /// Dynamically determines the path from the current node to the root
        /// based on current path and the site map.
        /// </summary>
        ///
        /// <remarks>
        /// <para>
        /// This method will retrieve the site map using <see cref="ISiteDiscovery"/> implementation
        /// and cache the site map in the page's Session with the key <see cref="CURRENT_SITEMAP"/>.
        /// Internally this method maintains a reverse site-map mapping in
        /// <see cref="CURRENT_SITEMAP_REVERSE"/>.
        /// It will then inspect if the node is in the current path (user goes back to previous node).
        /// Otherwise, it will check whether the current node is directly reacheable
        /// from the last node (using the site map information).
        /// Failing all above, it'll run a BFS to find the shortest path from this node
        /// to the current node and merge the found path with current path.
        /// If several path exists, the path with the most commonality with the current path is used.
        /// Finally if no shortest path is found, this is assumed a new dynamic node
        /// and will be added to current path (if exists).
        /// If no current path and no shortest path, then we return null because we have no idea
        /// where the user comes from.
        /// </para>
        /// <para>
        /// Note that the <see cref="ISiteDiscovery"/> is created using object factory
        /// with the namespace of <c>typeof(DynamicPathDiscovery).FullName</c> and key of
        /// <c>ISiteDiscovery</c>. For <see cref="XmlPathDiscovery"/>, the constructor which
        /// takes a filepath will need to be given an absolute path, since we cannot
        /// pass the Page Context via the interface to remap a relative path.
        /// </para>
        /// </remarks>
        ///
        /// <param name="currentNode">
        /// The current node.
        /// </param>
        /// <param name="page">
        /// The page context to save/get cached site map.
        /// </param>
        ///
        /// <returns>
        /// A path from root node (0th index) to current node ((n-1)th index)
        /// or null if no path can be found and no current path.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any argument is null or currentNode contains null Url (default initialization).
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// Any error occurs finding the path.
        /// </exception>
        public BreadCrumbNode[] GetPath(BreadCrumbNode currentNode, Page page)
        {
            if (currentNode.Url == null)
            {
                throw new ArgumentNullException("currentNode", "'currentNode.Url' cannot be null");
            }
            if (page == null)
            {
                throw new ArgumentNullException("page", "'page' cannot be null");
            }

            try
            {
                // get (reverse) site map from Session or create a new one (using ISiteDiscovery)
                // this will make sure Session has SiteMap & ReverseSiteMap
                IDictionary reverseSiteMap = GetReverseSiteMap(page);

                // get current path (could be null)
                NodePair[] curPath = GetCurrentPath(page);

                // 1. check current node on current path
                BreadCrumbNode[] newPath = CheckOnCurrent(currentNode, curPath, page);
                if (newPath != null)
                {
                    return newPath;
                }

                // 2. check current node is a direct child of last node
                newPath = CheckChildOfLastNode(currentNode, curPath, page);
                if (newPath != null)
                {
                    return newPath;
                }

                // 3. run shortest path from root to current node
                // result will be saved to ShortestPathResult instance
                // see OnRootFound() for detail algorithm
                object result = new ShortestPathResult();
                ((ShortestPathResult) result).CurrentNode = currentNode;
                ((ShortestPathResult) result).CurrentPath = curPath;

                PathAlg.FindShortestToRoot(
                    currentNode, (IDictionary) page.Session[CURRENT_SITEMAP_REVERSE],
                    new RootFound(OnRootFound), ref result);

                // do we find any shortest path?
                ShortestPathResult r = (ShortestPathResult) result;
                if (r.BestCommonCount != -1)
                {
                    // BestPath contains the best path from root to current node
                    NodePair[] newPairPath = (NodePair[]) r.BestPath.ToArray(typeof(NodePair));

                    // update state
                    page.Session[CURRENT_PATH] = newPairPath;

                    // build a new BreadCrumbNode[] array to be returned
                    return BuildBreadCrumbNode(newPairPath);
                }

                // 4. no shortest path found
                // if we have no current path either, we have no idea where we are
                if (curPath == null)
                {
                    return null;
                }

                // we hit a new dynamic link, just add it to the current path
                // copy all
                NodePair[] newDynamicPath = new NodePair[curPath.Length + 1];
                curPath.CopyTo(newDynamicPath, 0);

                // use current node as last node
                newDynamicPath[curPath.Length] = new NodePair(currentNode, null);

                // update state
                page.Session[CURRENT_PATH] = newDynamicPath;

                // build a new BreadCrumbNode[] array to be returned
                return BuildBreadCrumbNode(newDynamicPath);
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException(string.Format(
                    "Error retrieving path dynamically for url {0}", currentNode.Url), e);
            }
        }

        /// <summary>
        /// Gets the reverse site map from the Page's Session or
        /// create a new one from ISiteDiscovery implementation.
        /// After calling this function the Session will have both the sitemap
        /// and reverse sitemap cached.
        /// </summary>
        ///
        /// <param name="page">
        /// The page context to retrieve/save site map and its reverse site map.
        /// </param>
        ///
        /// <returns>
        /// The reverse site map.
        /// </returns>
        ///
        /// <exception cref="BreadCrumbException">
        /// Any error procesing the site map.
        /// </exception>
        private IDictionary GetReverseSiteMap(Page page)
        {
            try
            {
                // retrieve site map from the session
                object siteMap = page.Session[CURRENT_SITEMAP];
                if ((siteMap == null) || (!(siteMap is IDictionary)))
                {
                    // if not exist, create site map using ISiteDiscovery (and its reverse)
                    ISiteDiscovery siteDisc = (ISiteDiscovery) Helper.CreateInstance(
                        null, GetType().FullName, typeof(ISiteDiscovery).Name, typeof(ISiteDiscovery));
                    IDictionary newSiteMap = siteDisc.GetSiteMap();

                    page.Session[CURRENT_SITEMAP] = newSiteMap;
                    page.Session[CURRENT_SITEMAP_REVERSE] = PathAlg.CreateReverse(newSiteMap);
                }
                else
                {
                    // make sure a reverse site map exist
                    object reverseSiteMap = page.Session[CURRENT_SITEMAP_REVERSE];
                    if ((reverseSiteMap == null) || (!(reverseSiteMap is IDictionary)))
                    {
                        page.Session[CURRENT_SITEMAP_REVERSE] = PathAlg.CreateReverse((IDictionary) siteMap);
                    }
                }

                // return the reverse site map
                return (IDictionary) page.Session[CURRENT_SITEMAP_REVERSE];
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error processing sitemap", e);
            }
        }

        /// <summary>
        /// Gets the current path from the page's Session.
        /// If it's invalid, it'll be reset to null.
        /// </summary>
        ///
        /// <param name="page">
        /// The page context to retrieve/save current path.
        /// </param>
        ///
        /// <returns>
        /// The current path or null if it doesn't exist or invalid.
        /// </returns>
        ///
        /// <exception cref="BreadCrumbException">
        /// Any error retrieving the current path.
        /// </exception>
        private NodePair[] GetCurrentPath(Page page)
        {
            try
            {
                // retrieve current path from the session
                object path = page.Session[CURRENT_PATH];
                if (!(path is NodePair[]))
                {
                    // reset it
                    path = null;
                    page.Session[CURRENT_PATH] = path;
                }

                return (NodePair[]) path;
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error retrieving current path from session", e);
            }
        }

        /// <summary>
        /// Checks whether the current node is on the current path.
        /// </summary>
        ///
        /// <param name="curNode">
        /// The current node to check.
        /// </param>
        /// <param name="curPath">
        /// The current path to check against.
        /// This could be null and null will be returned as well.
        /// </param>
        /// <param name="page">
        /// The page context to save new current path to (if the current node is on the current path).
        /// </param>
        ///
        /// <returns>
        /// null if it's not, otherwise a new BreadCrumbNode[] representing a new truncated
        /// current path is returned.
        /// </returns>
        ///
        /// <exception cref="BreadCrumbException">
        /// Any error checking current node on the current path.
        /// </exception>
        private BreadCrumbNode[] CheckOnCurrent(
            BreadCrumbNode curNode, NodePair[] curPath, Page page)
        {
            // no curPath to check against
            if (curPath == null)
            {
                return null;
            }

            try
            {
                // iterate forward, find node in curPath that matches current node
                int foundIdx = -1;
                for (int i = 0; i < curPath.Length; i++)
                {
                    if (curPath[i].NodeMatcher != null)
                    {
                        // use the INodeMatcher
                        if (curPath[i].NodeMatcher.Matches(curNode))
                        {
                            foundIdx = i;
                            break;
                        }
                    }
                    else
                    {
                        // check against node url
                        if (curPath[i].Node.Url == curNode.Url)
                        {
                            foundIdx = i;
                            break;
                        }
                    }
                }

                // if we found it
                if (foundIdx != -1)
                {
                    // update state with a new NodePair[] of new length
                    // copy all, except the last node
                    NodePair[] newPath = new NodePair[foundIdx + 1];
                    Array.Copy(curPath, newPath, foundIdx);

                    // use current node as last idx
                    newPath[foundIdx] = new NodePair(curNode, curPath[foundIdx].NodeMatcher);

                    // update state
                    page.Session[CURRENT_PATH] = newPath;

                    // build a new BreadCrumbNode[] array to be returned
                    return BuildBreadCrumbNode(newPath);
                }

                return null;
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error checking current node on current path", e);
            }
        }

        /// <summary>
        /// Checks whether the current node is a direct child of the last node.
        /// </summary>
        ///
        /// <param name="curNode">
        /// The current node to check.
        /// </param>
        /// <param name="curPath">
        /// The current path to check against the last node.
        /// This could be null and null will be returned as well.
        /// </param>
        /// <param name="page">
        /// The page context to save new current path to
        /// (if the current node is a direct child of the last node).
        /// </param>
        ///
        /// <returns>
        /// null if it's not, otherwise a new BreadCrumbNode[] representing a new
        /// current path is returned.
        /// </returns>
        ///
        /// <exception cref="BreadCrumbException">
        /// Any error checking current node is direct child of last node on the current path.
        /// </exception>
        private BreadCrumbNode[] CheckChildOfLastNode(
            BreadCrumbNode curNode, NodePair[] curPath, Page page)
        {
            // no curPath to check against
            if (curPath == null)
            {
                return null;
            }

            try
            {
                // get last non-null node on current path
                INodeMatcher lastNode = null;
                for (int i = curPath.Length - 1; i >= 0; i--)
                {
                    if (curPath[i].NodeMatcher != null)
                    {
                        lastNode = curPath[i].NodeMatcher;
                        break;
                    }
                }
                if (lastNode == null)
                {
                    // can't find any node matcher
                    return null;
                }

                // get children of last node matcher from site map mapping
                IDictionary siteMap = (IDictionary) page.Session[CURRENT_SITEMAP];
                IList children = (IList) siteMap[lastNode];
                if ((children == null) || (children.Count == 0))
                {
                    // no child to check
                    return null;
                }

                // iterate all children and try to match the node
                foreach (INodeMatcher child in children)
                {
                    if (child.Matches(curNode))
                    {
                        // got a match, simply add this node to current path:

                        // update state with a new NodePair[] of new length
                        // copy all
                        NodePair[] newPath = new NodePair[curPath.Length + 1];
                        curPath.CopyTo(newPath, 0);

                        // use current node as last node
                        newPath[curPath.Length] = new NodePair(curNode, child);

                        // update state
                        page.Session[CURRENT_PATH] = newPath;

                        // build a new BreadCrumbNode[] array to be returned
                        return BuildBreadCrumbNode(newPath);
                    }
                }

                // no child match
                return null;
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error checking current node on current path", e);
            }
        }

        /// <summary>
        /// Builds a BreadCrumbNode[] array from the given NodePair[] array.
        /// </summary>
        ///
        /// <param name="pairs">
        /// The NodePair whose Node to copy.
        /// </param>
        ///
        /// <returns>
        /// A BreadCrumbNode[] of all Node in the given NodePair[].
        /// </returns>
        private BreadCrumbNode[] BuildBreadCrumbNode(NodePair[] pairs)
        {
            // build a new BreadCrumbNode[] array to be returned
            BreadCrumbNode[] result = new BreadCrumbNode[pairs.Length];

            // copy all Node
            for (int i = 0; i < pairs.Length; i++)
            {
                result[i] = pairs[i].Node;
            }
            return result;
        }

        /// <summary>
        /// The delegate that is called by PathAlg.FindShortestToRoot
        /// when a path to root is found.
        /// If we have no current path, this will be the shortest path
        /// and we can stop searching.
        /// If we have a current path, we'll take the one with the most similarity,
        /// merge it with the current path, and continue searching more.
        /// </summary>
        ///
        /// <param name="path">
        /// IList of INodeMatcher representing the path to root
        /// (first element is the current node, last element is root node).
        /// </param>
        /// <param name="result">
        /// A reference of ShortestPathResult instance to save result to.
        /// </param>
        ///
        /// <returns>
        /// Always false to stop searching if we have no current path.
        /// If we have current path, we'll merge by comparing best path with this path
        /// and get the one with the most commonality.
        /// </returns>
        private bool OnRootFound(IList path, ref object result)
        {
            // reverse path so that 0-th element is root
            IList reversedPath = new ArrayList();
            for (int i = path.Count - 1; i >= 0; i--)
            {
                reversedPath.Add(path[i]);
            }
            path = reversedPath;

            ShortestPathResult r = (ShortestPathResult) result;

            // no current path, this will be the best path and we're done
            if (r.CurrentPath == null)
            {
                r.BestPath = BuildNodePair(path);
                r.BestCommonCount = r.BestPath.Count - 1;  // don't include current node

                // replace last element with current node
                r.BestPath[r.BestPath.Count - 1] = new NodePair(
                    r.CurrentNode, (INodeMatcher) path[path.Count - 1]);

                return false;
            }

            // check how many common entry between CurrentPath and this shortest path
            int common = 0;
            while ((common < r.CurrentPath.Length) && (common < path.Count))
            {
                INodeMatcher matcher = r.CurrentPath[common].NodeMatcher;
                if ((matcher == null) || (!matcher.Equals((INodeMatcher) path[common])))
                {
                    // stop searching as soon as it doesn't match or hit a dynamic node
                    break;
                }
                else
                {
                    common++;
                }
            }

            // don't include current node
            if (common == path.Count)
            {
                common--;
            }

            // if we have more commonality, we have new best
            if (common > r.BestCommonCount)
            {
                // merge path with CurrentPath by
                // create a new path containing common element + all remaining path to currentNode
                ArrayList newCommonPath = new ArrayList();    // IList of NodePair
                for (int i = 0; i < common; i++)
                {
                    newCommonPath.Add(r.CurrentPath[i]);
                }
                // + all remaining path to currentNode
                for (int i = common; i < path.Count; i++)
                {
                    INodeMatcher matcher = (INodeMatcher) path[i];

                    // use CurrentNode if last element or create a new BreadCrumbNode otherwise
                    BreadCrumbNode bcNode = ((i == path.Count - 1) ?
                        r.CurrentNode : new BreadCrumbNode(matcher.Title, matcher.Url));
                    newCommonPath.Add(new NodePair(bcNode, matcher));
                }

                r.BestCommonCount = common;
                r.BestPath = newCommonPath;
            }

            // search more
            return true;
        }

        /// <summary>
        /// Builds ArrayList of NodePair from the given IList of INodeMatcher.
        /// </summary>
        ///
        /// <param name="matchers">
        /// The IList of INodeMatcher.
        /// </param>
        ///
        /// <returns>
        /// An ArrayList of INodePair.
        /// </returns>
        private ArrayList BuildNodePair(IList matchers)
        {
            ArrayList result = new ArrayList();
            foreach (INodeMatcher match in matchers)
            {
                result.Add(new NodePair(
                    new BreadCrumbNode(match.Title, match.Url), match));
            }
            return result;
        }

    }
}
