/*
 * PathAlg.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System.Collections;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// A delegate declaration of the method that would be called by PathAlg.FindShortestToRoot
    /// when it finds a root node.
    /// </summary>
    ///
    /// <param name="path">
    /// The IList containing INodeMatcher path.
    /// First element will be the start node, last element will be the root node.
    /// </param>
    /// <param name="userData">
    /// A reference to a user supplied object, which was passed to PathAlg.FindShortestToRoot.
    /// </param>
    ///
    /// <returns>
    /// Return true if the algorithm should continue search; false to stop searching.
    /// </returns>
    internal delegate bool RootFound(IList path, ref object userData);

    /// <summary>
    /// Helper class for path related algorithm.
    /// </summary>
    internal sealed class PathAlg
    {
        /// <summary>
        /// Prevents this class from being instantiated.
        /// </summary>
        private PathAlg()
        {
        }

        /// <summary>
        /// Creates a reverse map (children to parent mapping) given
        /// the parent to children mapping.
        /// </summary>
        ///
        /// <param name="map">
        /// The parent to children IList mapping.
        /// </param>
        ///
        /// <returns>
        /// The reverse children to parent IList mapping.
        /// </returns>
        internal static IDictionary CreateReverse(IDictionary map)
        {
            IDictionary reverse = new Hashtable();
            foreach (DictionaryEntry entry in map)
            {
                // reverse the parent key
                object parent = entry.Key;
                AddIListValue(reverse, parent, null);

                // reverse each child element
                foreach (object child in ((IList) entry.Value))
                {
                    AddIListValue(reverse, child, parent);
                }
            }
            return reverse;
        }

        /// <summary>
        /// Finds shortest path from current node to the root node.
        /// </summary>
        ///
        /// <remarks>
        /// A BFS algorithm is used because it'll find shorter path first.
        /// Note that this algorithm won't invoke the delegate for a node that has been seen.
        /// For example if it has found B->A->root, then it won't attempt D->C->A->root because
        /// node A was already seen on the shorter path.
        /// </remarks>
        ///
        /// <param name="startNode">
        /// The current node.
        /// </param>
        /// <param name="map">
        /// The IDictionary of <see cref="INodeMatcher"/> child to
        /// IList of <see cref="INodeMatcher"/> parent.
        /// </param>
        /// <param name="rootFound">
        /// The delegate to invoke when a root node is found.
        /// The delegate should return false to continue searching the next shortest path,
        /// or return false to stop searching.
        /// </param>
        /// <param name="userData">
        /// A reference to the user data. This will be passed to the rootFound delegate
        /// and could be anything.
        /// </param>
        internal static void FindShortestToRoot(
            BreadCrumbNode startNode, IDictionary map, RootFound rootFound, ref object userData)
        {
            // the BFS queue
            // this will contain an IList of INodeMatcher representing the path from startNode
            Queue q = new Queue();

            // save visited/seen INodeMatcher
            // so that we don't visit same node twice or create endless loop
            IDictionary visited = new Hashtable();

            // find all INodeMatcher for this node (these are the initial states)
            foreach (INodeMatcher curMatcher in map.Keys)
            {
                if (curMatcher.Matches(startNode))
                {
                    // found a possible initial state, queue it
                    IList initialState = new ArrayList();
                    initialState.Add(curMatcher);
                    q.Enqueue(initialState);
                }
            }

            // process queue
            while (q.Count > 0)
            {
                IList curState = (IList) q.Dequeue();

                // the last node is the last element of current state
                INodeMatcher lastNode = (INodeMatcher) curState[curState.Count - 1];
                if (visited.Contains(lastNode))
                {
                    // skip this if we have been here in previously shorter cycle
                    continue;
                }
                visited.Add(lastNode, null);    // mark as visited

                // next state is all its parents
                IList parents = (IList) map[lastNode];

                // no more parents means we found the root!
                if (parents.Count == 0)
                {
                    // invoke the delegate and let it decide whether to continue searching
                    if (!rootFound(curState, ref userData))
                    {
                        // stop searching
                        break;
                    }
                }

                // queue all parents as next state
                foreach (INodeMatcher parent in parents)
                {
                    // create a new array reference
                    IList nextState = new ArrayList(curState);
                    nextState.Add(parent);
                    q.Enqueue(nextState);
                }
            }
        }

        /// <summary>
        /// Appends the key and value to the IDictionary.
        /// If the key doesn't exist, a new IList value is created.
        /// If it exists, the value is appended to the IList.
        /// </summary>
        ///
        /// <param name="dict">
        /// The IDictionary to add to.
        /// </param>
        /// <param name="key">
        /// The key to add.
        /// </param>
        /// <param name="val">
        /// The value to add. This could be null which means an empty IList is created
        /// if the key doesn't exist.
        /// </param>
        private static void AddIListValue(IDictionary dict, object key, object val)
        {
            // if key doesn't exist, create a new IList
            if (!dict.Contains(key))
            {
                dict[key] = new ArrayList();
            }

            // append to the IList value
            if (val != null)
            {
                ((IList) dict[key]).Add(val);
            }
        }
    }
}
