/*
 * AnchorPathWriter.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Web;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// Writes the node and title using the html anchor pattern of
    /// &lt;a href="node.Url"&gt;node.Title&lt;/a&gt;.
    /// </summary>
    ///
    /// <remarks>
    /// This path writer will be created by the Object Factory and utilized by the
    /// <see cref="BreadCrumbTrailControl"/> to write the current node/title using an html anchor.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class AnchorPathWriter : AbstractPathWriter
    {
        /// <summary>
        /// The onMouseOver attribute for the tag.
        /// </summary>
        private const string OnMouseOver = "onMouseOver";

        /// <summary>
        /// The onMouseOut attribute for the tag.
        /// </summary>
        private const string OnMouseOut = "onMouseOut";

        /// <summary>
        /// The value for onMouseOver and onMouseOut attribute.
        /// According to DOM reference, all HTML elements have a className
        /// property to set its class.
        /// </summary>
        private const string OnMouseStyle = "this.className='{0}'";

        /// <summary>
        /// Default title to render if the given node has null/empty trimmed title.
        /// </summary>
        private const string DefaultTitle = "Unknown";

        /// <summary>
        /// Initializes a new instance of the <see cref="AnchorPathWriter"/> class.
        /// </summary>
        public AnchorPathWriter()
        {
            // nothing to do (base() is automatically invoked)
        }

        /// <summary>
        /// Writes the specified node and style as a simple html anchor.
        /// </summary>
        ///
        /// <remarks>
        /// Please note that an empty (trimmed) string is equivalent to a null.
        /// This method will render the node into the following html: <c><![CDATA[
        /// <a href="node.Url" class="nodeStyle" onMouseOver="this.className='mouseOverStyle'"
        /// onMouseOut="this.className='nodeStyle'"/>node.Title</a>]]></c>.
        /// class attribute is added only if <paramref name="nodeStyle"/> is not null.
        /// onMouseOver attribute is added only if <paramref name="mouseOverStyle"/> is not null.
        /// onMouseOut attribute is added if onMouseOver was added. This attribute will
        /// reset the class to nodeStyle or empty (if it doesn't exist).
        /// If the node title doesn't exist or is an empty trimmed string, a default title
        /// of "Unknown" is used instead.
        /// </remarks>
        ///
        /// <param name="writer">
        /// A non-null writer to write to.
        /// </param>
        /// <param name="node">
        /// A non-null node to write.
        /// </param>
        /// <param name="nodeStyle">
        /// A style class to use for the node or null if no style should be used.
        /// </param>
        /// <param name="mouseOverStyle">
        /// The mouse over style class to use or null for none.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is null or <paramref name="node"/> has a null Url.
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// Any error occurs writing the node.
        /// </exception>
        protected override void WriteNode(
            HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle, string mouseOverStyle)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer", "'writer' cannot be null");
            }
            if (node.Url == null)
            {
                throw new ArgumentNullException("node", "'node.Url' cannot be null");
            }

            try
            {
                bool isNullNodeStyle = ((nodeStyle == null) || (nodeStyle.Trim().Length == 0));

                // render nodeStyle as class attribute
                if (!isNullNodeStyle)
                {
                    writer.AddAttribute(HtmlTextWriterAttribute.Class, nodeStyle);
                }

                // render onMouseOver and onMouseOut attributes
                if ((mouseOverStyle != null) && (mouseOverStyle.Trim().Length > 0))
                {
                    writer.AddAttribute(OnMouseOver, string.Format(OnMouseStyle, mouseOverStyle));
                    writer.AddAttribute(OnMouseOut, string.Format(OnMouseStyle,
                        (isNullNodeStyle ? "" : nodeStyle)));
                }

                // render href=node.Url atribute
                writer.AddAttribute(HtmlTextWriterAttribute.Href, node.Url);

                // render A tag, value is node.Title (use DefaultTitle if null or empty)
                writer.RenderBeginTag(HtmlTextWriterTag.A);
                string title = (((node.Title == null) || (node.Title.Trim().Length == 0)) ?
                    DefaultTitle : node.Title);
                writer.Write(HttpUtility.HtmlEncode(title));
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing node", e);
            }
        }
    }
}
