/*
 * OrderedListPathWriter.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// Writes the node and title using the ordered list pattern
    /// where each rendered node is a separate line item.
    /// </summary>
    ///
    /// <remarks>
    /// This path writer will be created by the Object Factory and utilized by the
    /// <see cref="BreadCrumbTrailControl"/> to write the current node/title using an ordered list.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class OrderedListPathWriter : AnchorPathWriter
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="OrderedListPathWriter"/> class.
        /// </summary>
        public OrderedListPathWriter()
        {
            // nothing to do (base() is automatically invoked)
        }

        /// <summary>
        /// Writes the beginning tag (&lt;ol&gt;) for the ordered list.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is a null reference.
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// An exception occurs writing the beginning tag for the ordered list.
        /// </exception>
        protected override void BeginPath(HtmlTextWriter writer)
        {
            // base checks null writer
            base.BeginPath(writer);

            try
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Ol);
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing beginning tag of ordered list", e);
            }
        }

        /// <summary>
        /// Write the specified node and style as a list item.
        /// </summary>
        ///
        /// <remarks>
        /// Please note that an empty (trimmed) string is equivalent to a null.
        /// This method will render the node into the following html: <c><![CDATA[
        /// <li><a href="node.Url" class="nodeStyle" onMouseOver="this.className='mouseOverStyle'"
        /// onMouseOut="this.className='nodeStyle'"/>node.Title</a></li>]]></c>.
        /// The A tag is rendered using <see cref="AnchorPathWriter.WriteNode"/>.
        /// </remarks>
        ///
        /// <param name="writer">
        /// A non-null writer to write to.
        /// </param>
        /// <param name="node">
        /// A non-null node to write.
        /// </param>
        /// <param name="nodeStyle">
        /// A style class to use for the node or null if no style should be used.
        /// </param>
        /// <param name="mouseOverStyle">
        /// The mouse over style class to use or null for none.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is null or <paramref name="node"/> has a null Url.
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// Any error occurs writing the node.
        /// </exception>
        protected override void WriteNode(
            HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle, string mouseOverStyle)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer", "'writer' cannot be null");
            }
            if (node.Url == null)
            {
                throw new ArgumentNullException("node", "'node.Url' cannot be null");
            }

            try
            {
                // <li> {base.WriteNode} </li>
                writer.RenderBeginTag(HtmlTextWriterTag.Li);
                base.WriteNode(writer, node, nodeStyle, mouseOverStyle);
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing node", e);
            }
        }

        /// <summary>
        /// Method is overridden to prevent the separator from being written.
        /// This method does nothing.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to use.
        /// </param>
        /// <param name="parent">
        /// A non-null parent control.
        /// </param>
        protected override void WriteSeparator(HtmlTextWriter writer, BreadCrumbTrailControl parent)
        {
            // do nothing
        }

        /// <summary>
        /// Writes the ending tag (&lt;/ol&gt;) for the ordered list.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is a null reference.
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// An exception occurs writing the ending tag for the ordered list.
        /// </exception>
        protected override void EndPath(HtmlTextWriter writer)
        {
            // base checks null writer
            base.EndPath(writer);

            try
            {
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing ending tag of ordered list", e);
            }
        }
    }
}
