/*
 * MyPathWriter.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.Tests
{
    /// <summary>
    /// A concrete class derived from <see cref="AbstractPathWriter"/> so that it can be tested.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// This class provides a simple implementation of <see cref="WriteNode"/>.
    /// <see cref="WriteNode"/> will write the passed node's Title, style, and mouseoverstyle
    /// into the html writer so that it can be asserted.
    /// It also overrides <see cref="BeginPath"/> and <see cref="EndPath"/> by writing some dummy strings
    /// into the html writer so that it can be asserted that the methods are invoked in the correct order.
    /// This also adds <see cref="MyWriteSeparator"/>, <see cref="MyBeginPath"/>, <see cref="MyEndPath"/>
    /// which provides a public access to the protected <see cref="AbstractPathWriter.WriteSeparator"/>,
    /// <see cref="AbstractPathWriter.BeginPath"/>, <see cref="AbstractPathWriter.EndPath"/>
    /// so that they can be asserted easily.
    /// </para>
    /// <para>
    /// The overridden method may also throw an exception if initialized using ctor
    /// that accepts a string.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class MyPathWriter : AbstractPathWriter
    {
        /// <summary>
        /// The html output of the overridden <see cref="BeginPath"/> method.
        /// </summary>
        public const string BeginPathHtml = "BeginPath";

        /// <summary>
        /// The html output of the overridden <see cref="EndPath"/> method.
        /// </summary>
        public const string EndPathHtml = "EndPath";

        /// <summary>
        /// The html output of the <see cref="WriteNode"/> method.
        /// {0}, {1}, {2} is placeholder for the corresponding parameter values.
        /// </summary>
        public const string NodeHtml = "<Node node=\"{0}\" nodeStyle=\"{1}\" mouseOverStyle=\"{2}\"></Node>";

        /// <summary>
        /// Checked by various methods to throw exceptions.
        /// Initialized in ctor(string). Default to empty string.
        /// </summary>
        private readonly string methodException = "";

        /// <summary>
        /// Initializes a new instance of the <see cref="MyPathWriter"/> class.
        /// </summary>
        public MyPathWriter()
        {
            // do nothing
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MyPathWriter"/> class
        /// that will throw an exception depending on the given string.
        /// </summary>
        ///
        /// <param name="methodException">
        /// If it is "BeginPath"/"EndPath", then the corresponding method will throw exception
        /// when invoked.
        /// Otherwise, this string is used in WriteNode which will throw an exception
        /// if the node's title match this string.
        /// </param>
        public MyPathWriter(string methodException)
        {
            this.methodException = methodException;
        }

        /// <summary>
        /// Overrides default BeginPath by writing "BeginPath" to the html writer.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to write to.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is a null reference.
        /// </exception>
        /// <exception cref="InvalidOperationException">
        /// If methodException equals to "BeginPath".
        /// </exception>
        protected override void BeginPath(HtmlTextWriter writer)
        {
            if (methodException == BeginPathHtml)
            {
                throw new InvalidOperationException("BeginPath throw exception");
            }
            base.BeginPath(writer);
            writer.Write(BeginPathHtml);
        }

        /// <summary>
        /// Writes the tag &lt;Node/&gt; with node's title, style, and mouseoverstyle as its attributes.
        /// The attribute name will be the parameter name itself.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to write to.
        /// </param>
        /// <param name="node">
        /// A non-null node to write.
        /// </param>
        /// <param name="nodeStyle">
        /// A style class to use for the node or null if no style should be used.
        /// </param>
        /// <param name="mouseOverStyle">
        /// The mouse over style class to use or null for none.
        /// </param>
        ///
        /// <exception cref="InvalidOperationException">
        /// If methodException equals to the node's Title.
        /// </exception>
        protected override void WriteNode(
            HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle, string mouseOverStyle)
        {
            if (methodException == node.Title)
            {
                throw new InvalidOperationException("WriteNode throw exception");
            }
            // <Node node=nodeTitle nodeStyle=... mouseOverStyle=.../>
            writer.AddAttribute("node", node.Title);
            writer.AddAttribute("nodeStyle", nodeStyle);
            writer.AddAttribute("mouseOverStyle", mouseOverStyle);
            writer.RenderBeginTag("Node");
            writer.RenderEndTag();
        }

        /// <summary>
        /// Overrides default EndPath by writing "EndPath" to the html writer.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to write to.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is a null reference.
        /// </exception>
        /// <exception cref="InvalidOperationException">
        /// If methodException equals to "EndPath".
        /// </exception>
        protected override void EndPath(HtmlTextWriter writer)
        {
            if (methodException == EndPathHtml)
            {
                throw new InvalidOperationException("EndPath throw exception");
            }
            base.EndPath(writer);
            writer.Write(EndPathHtml);
        }

        /// <summary>
        /// Simply forwards the call to the protected <see cref="AbstractPathWriter.WriteSeparator"/>.
        /// </summary>
        ///
        /// <param name="writer">
        ///  A non-null writer to use.
        ///  </param>
        /// <param name="parent">
        /// A non-null parent control to retrieve PathSeparator and PathSeparatorStyle.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        public void MyWriteSeparator(HtmlTextWriter writer, BreadCrumbTrailControl parent)
        {
            WriteSeparator(writer, parent);
        }

        /// <summary>
        /// Simply forwards the call to the protected <see cref="AbstractPathWriter.BeginPath"/>.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to write to.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is a null reference.
        /// </exception>
        public void MyBeginPath(HtmlTextWriter writer)
        {
            BeginPath(writer);
        }

        /// <summary>
        /// Simply forwards the call to the protected <see cref="AbstractPathWriter.EndPath"/>.
        /// </summary>
        ///
        /// <param name="writer">
        /// A non-null writer to write to.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="writer"/> is a null reference.
        /// </exception>
        public void MyEndPath(HtmlTextWriter writer)
        {
            EndPath(writer);
        }
    }
}
