/*
 * TestHttpRequestNodeDiscovery.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.Web;
using System.Web.UI;
using NUnit.Framework;
using NUnit.Extensions.Asp;
using NUnit.Extensions.Asp.AspTester;
using TopCoder.Web.UI.WebControl.BreadCrumb.Tests;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.NodeDiscovery.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="HttpRequestNodeDiscovery"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// Since the class requires a valid <see cref="HttpRequest"/>,
    /// an actual IIS server is used.
    /// The aspx file TestHttpRequestNodeDiscovery.aspx is used to test
    /// the class functionality. The aspx page will instantiate an instance
    /// of <see cref="HttpRequestNodeDiscovery"/> and invoke the
    /// <see cref="HttpRequestNodeDiscovery.GetCurrentNode"/> method.
    /// The first parameter will be the query string value of 'title' if it exists.
    /// The second parameter is the page instance itself.
    /// The output of the method, which is a <see cref="BreadCrumbNode"/>,
    /// is then assigned to the asp label, which will be asserted here.
    /// Note that the output of the method will be html encoded
    /// otherwise NUnitAsp will emit warnings about unrecognized element.
    /// </remarks>
    [TestFixture]
    public class TestHttpRequestNodeDiscovery : WebFormTestCase
    {
        /// <summary>
        /// The path of the aspx file.
        /// </summary>
        private const string TestPath = TestCommon.AppPath + "/TestHttpRequestNodeDiscovery.aspx";

        /// <summary>
        /// The url of the aspx used to test HttpRequestNodeDiscovery.
        /// </summary>
        private const string TestUrl = TestCommon.IisServer + TestPath;

        /// <summary>
        /// Represents the lblTitle label. Initialized in SetUp.
        /// </summary>
        private LabelTester lblTitle;

        /// <summary>
        /// Represents the lblUrl label. Initialized in SetUp.
        /// </summary>
        private LabelTester lblUrl;

        /// <summary>
        /// Initializes the control testers.
        /// </summary>
        protected override void SetUp()
        {
            lblTitle = new LabelTester("lblTitle", CurrentWebForm);
            lblUrl = new LabelTester("lblUrl", CurrentWebForm);
        }

        /// <summary>
        /// Tests requesting the page without any query string.
        /// The resulting title will be null and the url will have no query string as well.
        /// </summary>
        [Test]
        public void TestNoData()
        {
            Browser.GetPage(TestUrl);

            // assert Title is empty (converted to null) and Url is the path without any query string
            AssertEquals("Null title", ":null:", lblTitle.Text);
            AssertEquals("Url contains the path", TestPath, lblUrl.Text);
        }

        /// <summary>
        /// Tests requesting the page with some unsorted query strings.
        /// The query string key should be sorted when returned.
        /// </summary>
        [Test]
        public void TestQueryString()
        {
            // get the page with some query parameters
            Browser.GetPage(TestUrl + "?title=TestTitle&bb=123&b1=aa&aa=zz");

            // assert Title and Url
            AssertEquals("Title should exist", "TestTitle", lblTitle.Text);
            AssertEquals("Url contains the sorted query string",
                HttpUtility.HtmlEncode(TestPath + "?aa=zz&b1=aa&bb=123&title=TestTitle"),
                lblUrl.Text);
        }

        /// <summary>
        /// Tests requesting the page with some query strings that contains some url encoded name/value.
        /// The query string key should be returned decoded although the request is url-encoded.
        /// </summary>
        [Test]
        public void TestQueryStringUrlEncoded()
        {
            // get the page with some url encoded query parameters (title is an empty string)
            Browser.GetPage(TestUrl + string.Format(
                "?title=+&{0}={1}&{2}={3}",
                HttpUtility.UrlEncode("p 2"), HttpUtility.UrlEncode("<> &="),
                HttpUtility.UrlEncode("p 1"), HttpUtility.UrlEncode("#$%")));

            // assert Title and Url
            AssertEquals("Title should be null", ":null:", lblTitle.Text);
            AssertEquals("Url contains decoded query string",
                HttpUtility.HtmlEncode(TestPath + "?p 1=#$%&p 2=<> &=&title= "),
                lblUrl.Text);
        }

        /// <summary>
        /// Tests GetCurrentNode given null page.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullPage()
        {
            new HttpRequestNodeDiscovery().GetCurrentNode("title", null);
        }

        /// <summary>
        /// Tests GetCurrentNode given a Page with no Request instance.
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestNullPageRequest()
        {
            new HttpRequestNodeDiscovery().GetCurrentNode("title", new Page());
        }
    }
}
