/*
 * TestXmlPathDiscovery.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.Collections;
using NUnit.Framework;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="XmlPathDiscovery"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// This tests the GetSiteMap method and constructor.
    /// GetPath is tested in <see cref="TestXmlPathDiscoveryGetPath"/>
    /// as that method is tested using NUnitAsp.
    /// </remarks>
    [TestFixture]
    public class TestXmlPathDiscovery
    {
        /// <summary>
        /// Tests static constant values.
        /// </summary>
        [Test]
        public void TestStatic()
        {
            Assert.AreEqual("TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.XmlPathDiscovery.siteMap",
                XmlPathDiscovery.CURRENT_SITEMAP, "Static constant value is incorrect");

            Assert.AreEqual("TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.XmlPathDiscovery.siteMap.reverse",
                XmlPathDiscovery.CURRENT_SITEMAP_REVERSE, "Static reverse constant value is incorrect");
        }

        /// <summary>
        /// Tests constructor(string) given null filename.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorStringNull()
        {
            new XmlPathDiscovery(null);
        }

        /// <summary>
        /// Tests constructor(string) given empty filename.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringEmpty()
        {
            new XmlPathDiscovery(" ");
        }

        /// <summary>
        /// Tests constructor(string) and GetSiteMap given invalid filename.
        /// The constructor should have no problem, but BreadCrumbException
        /// should be thrown by GetSiteMap.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestCtorStringInvalid()
        {
            new XmlPathDiscovery("NotExists.xml").GetSiteMap();
        }

        /// <summary>
        /// Tests GetSiteMap and constructor(string) accuracy with a map file.
        /// The map has 7 unique nodes, one of the node has 2 parents.
        /// </summary>
        [Test]
        public void TestGetSiteMapCtorString()
        {
            XmlPathDiscovery path = new XmlPathDiscovery(@"..\..\test_files\TestXmlPathDiscovery.xml");
            IDictionary result = path.GetSiteMap();

            Assert.AreEqual(7, result.Count, "7 unique nodes defined");
            AssertMapEntry(new string[,] {
                {"node 1.1", "/root/1.1", "/1.1.*"},
                {"node 1.2", "/root/1.2", "/1.2?.*"}}, result, "root 1", "/root_1", "/");
            AssertMapEntry(new string[0,0], result, "node 1.1", "/root/1.1", "/1.1.*");
            AssertMapEntry(new string[,] {
                {"leaf 1.2.3", "/leaf", "/leafUrl?.*"}}, result, "node 1.2", "/root/1.2", "/1.2?.*");
            AssertMapEntry(new string[0,0], result, "leaf 1.2.3", "/leaf", "/leafUrl?.*");

            AssertMapEntry(new string[,] {
                {"node 2.1", "/root/2.1", "/2.1.*"}}, result, "root 2", "/root_2", "/root2");
            AssertMapEntry(new string[,] {
                {"leaf 1.2.3", "/leaf", "/leafUrl?.*"},
                {"leaf 2.1.1", "/leaf2", "/leaf2Url?.*"}}, result, "node 2.1", "/root/2.1", "/2.1.*");
        }

        /// <summary>
        /// Tests GetSiteMap and constructor(string, true) accuracy with a map file.
        /// This is similar to the test using constructor(string).
        /// </summary>
        [Test]
        public void TestGetSiteMapCtorStringTrue()
        {
            XmlPathDiscovery path = new XmlPathDiscovery(@"..\..\test_files\TestXmlPathDiscovery.xml", true);
            IDictionary result = path.GetSiteMap();

            Assert.AreEqual(7, result.Count, "7 unique nodes defined");
            AssertMapEntry(new string[,] {
                {"node 1.1", "/root/1.1", "/1.1.*"},
                {"node 1.2", "/root/1.2", "/1.2?.*"}}, result, "root 1", "/root_1", "/");
            AssertMapEntry(new string[0,0], result, "node 1.1", "/root/1.1", "/1.1.*");
            AssertMapEntry(new string[,] {
                {"leaf 1.2.3", "/leaf", "/leafUrl?.*"}}, result, "node 1.2", "/root/1.2", "/1.2?.*");
            AssertMapEntry(new string[0,0], result, "leaf 1.2.3", "/leaf", "/leafUrl?.*");

            AssertMapEntry(new string[,] {
                {"node 2.1", "/root/2.1", "/2.1.*"}}, result, "root 2", "/root_2", "/root2");
            AssertMapEntry(new string[,] {
                {"leaf 1.2.3", "/leaf", "/leafUrl?.*"},
                {"leaf 2.1.1", "/leaf2", "/leaf2Url?.*"}}, result, "node 2.1", "/root/2.1", "/2.1.*");
            AssertMapEntry(new string[0,0], result, "leaf 2.1.1", "/leaf2", "/leaf2Url?.*");
        }

        /// <summary>
        /// Tests GetSiteMap and constructor(string, false) accuracy with a string source.
        /// The map also contains a loop.
        /// </summary>
        [Test]
        public void TestGetSiteMapCtorStringFalse()
        {
            XmlPathDiscovery path = new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>

                <BreadCrumb version=""v2"">

                    <!-- 3 unique nodes with a loop -->
                    <bc_node title=""1"" url=""url1"" pattern=""/pat1"">
                        <bc_node title=""1-1"" url=""url1-1"" pattern=""/pat1-1""/>
                        <bc_node title=""1-0"" url=""url1-0"" pattern=""/pat1-0"">
                            <bc_node title=""1"" url=""url1"" pattern=""/pat1""/>
                        </bc_node>
                    </bc_node>
                </BreadCrumb>", false);

            IDictionary result = path.GetSiteMap();

            Assert.AreEqual(3, result.Count, "3 unique nodes with a loop");
            AssertMapEntry(new string[,] {
                {"1-1", "url1-1", "/pat1-1"},
                {"1-0", "url1-0", "/pat1-0"}}, result, "1", "url1", "/pat1");
            AssertMapEntry(new string[0,0], result, "1-1", "url1-1", "/pat1-1");
            AssertMapEntry(new string[,] {
                {"1", "url1", "/pat1"}}, result, "1-0", "url1-0", "/pat1-0");
        }

        /// <summary>
        /// Tests GetSiteMap with a map where a node is defined in 2 different places.
        /// The first one has children, the second one is childless.
        /// </summary>
        [Test]
        public void TestGetSiteMapFirstHasChildren()
        {
            XmlPathDiscovery path = new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>

                <BreadCrumb version=""v2"">
                    <bc_node title=""1"" url=""url1"" pattern=""/pat1"">
                        <bc_node title=""1-1"" url=""url1-1"" pattern=""/pat1-1""/>
                    </bc_node>
                    <!-- redefine without any children -->
                    <bc_node title=""1"" url=""url1"" pattern=""/pat1""/>
                </BreadCrumb>", false);

            IDictionary result = path.GetSiteMap();

            Assert.AreEqual(2, result.Count, "2 unique nodes");
            AssertMapEntry(new string[,] {
                {"1-1", "url1-1", "/pat1-1"}}, result, "1", "url1", "/pat1");
            AssertMapEntry(new string[0,0], result, "1-1", "url1-1", "/pat1-1");
        }

        /// <summary>
        /// Tests GetSiteMap with a map where a node is defined in 2 different places.
        /// The first one is childless, the second one defines the children.
        /// </summary>
        [Test]
        public void TestGetSiteMapSecondHasChildren()
        {
            XmlPathDiscovery path = new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>

                <BreadCrumb version=""v2"">
                    <bc_node title=""1"" url=""url1"" pattern=""/pat1""/>
                    <!-- redefine the children -->
                    <bc_node title=""1"" url=""url1"" pattern=""/pat1"">
                        <bc_node title=""1-1"" url=""url1-1"" pattern=""/pat1-1""/>
                    </bc_node>
                </BreadCrumb>", false);

            IDictionary result = path.GetSiteMap();

            Assert.AreEqual(2, result.Count, "2 unique nodes");
            AssertMapEntry(new string[,] {
                {"1-1", "url1-1", "/pat1-1"}}, result, "1", "url1", "/pat1");
            AssertMapEntry(new string[0,0], result, "1-1", "url1-1", "/pat1-1");
        }

        /// <summary>
        /// Tests GetSiteMap given minimal xml source.
        /// Only BreadCrumb tag is defined and no node is defined.
        /// </summary>
        [Test]
        public void TestGetSiteMapNoNode()
        {
            IDictionary result = new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>
                <BreadCrumb>
                </BreadCrumb>", false).GetSiteMap();

            Assert.AreEqual(0, result.Count, "No node defined");
        }

        /// <summary>
        /// Tests GetSiteMap given empty xml string source.
        /// Expecting BreadCrumbException because it's not a valid xml
        /// (missing root document element).
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapEmpty()
        {
            new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>", false).GetSiteMap();
        }

        /// <summary>
        /// Tests GetSiteMap given invalid xml source: an unknown element exists.
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapUnknown()
        {
            new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>
                <BreadCrumb version=""v2"">
                    <bc_node title=""1"" url=""url1"" pattern=""/pat1"">
                        <!-- unknown element -->
                        <bc_nodex title=""1-1"" url=""url1-1"" pattern=""/pat1-1""/>
                    </bc_node>
                </BreadCrumb>", false).GetSiteMap();
        }

        /// <summary>
        /// Tests GetSiteMap given invalid xml source: missing required title attribute
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapMissingTitle()
        {
            new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>
                <BreadCrumb version=""v2"">
                    <bc_node url=""url1"" pattern=""/pat1""/>
                </BreadCrumb>", false).GetSiteMap();
        }

        /// <summary>
        /// Tests GetSiteMap given invalid xml source: missing required url attribute
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapMissingUrl()
        {
            new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>
                <BreadCrumb version=""v2"">
                    <bc_node title=""1"" pattern=""/pat1""/>
                </BreadCrumb>", false).GetSiteMap();
        }

        /// <summary>
        /// Tests GetSiteMap given invalid xml source: missing required pattern attribute
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapMissingPattern()
        {
            new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>
                <BreadCrumb version=""v2"">
                    <bc_node title=""1"" url=""url1""/>
                </BreadCrumb>", false).GetSiteMap();
        }

        /// <summary>
        /// Tests GetSiteMap given invalid xml source:
        /// a node is defined twice with children.
        /// Only one should define the children, the other should be childless.
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetSiteMapDifferentNodeChildren()
        {
            new XmlPathDiscovery(
                @"<?xml version=""1.0"" encoding=""UTF-8""?>
                <BreadCrumb version=""v2"">
                    <bc_node title=""1"" url=""url1"" pattern=""pat1"">
                        <bc_node title=""2"" url=""url2"" pattern=""pat2""/>
                    </bc_node>
                    <bc_node title=""1"" url=""url1"" pattern=""pat1"">
                        <!-- redefine children of node 1 -->
                        <bc_node title=""2"" url=""url2"" pattern=""pat2""/>
                    </bc_node>
                </BreadCrumb>", false).GetSiteMap();
        }

        /// <summary>
        /// Asserts a single entry of the IDictionary returned by GetSiteMap().
        /// </summary>
        ///
        /// <param name="expected">
        /// The expected value of the given dictionary key.
        /// It must be formatted as string[n,3] where n is the expected IList count,
        /// [i,0] is the expected title, [i,1] is the expected url, [i,2] is the expected pattern.
        /// </param>
        /// <param name="actual">
        /// The dictionary whose entry value to assert.
        /// </param>
        /// <param name="parentTitle">
        /// The Title of the key whose value to assert.
        /// </param>
        /// <param name="parentUrl">
        /// The Url of the key whose value to assert.
        /// </param>
        /// <param name="parentPattern">
        /// The Pattern of the key whose value to assert.
        /// </param>
        public void AssertMapEntry(string[,] expected, IDictionary actual,
            string parentTitle, string parentUrl, string parentPattern)
        {
            INodeMatcher key = new RegexNodeMatcher(parentTitle, parentUrl, parentPattern);

            // get the children IList
            IList children = (IList) actual[key];
            Assert.IsNotNull(children, string.Format(
                "No entry found for {0} - {1} - {2}", parentTitle, parentUrl, parentPattern));

            // assert the IList length first, then assert each entry
            int expectedCount = expected.GetLength(0);
            Assert.AreEqual(expectedCount, children.Count, "Different number of element count");
            for (int i = 0; i < expectedCount; i++)
            {
                RegexNodeMatcher child = (RegexNodeMatcher) children[i];
                Assert.AreEqual(expected[i, 0], child.Title, "Child " + i + "'s Title");
                Assert.AreEqual(expected[i, 1], child.Url, "Child " + i + "'s Url");
                Assert.AreEqual(expected[i, 2], child.NodePattern, "Child " + i + "'s Pattern");
            }
        }
    }
}

