/*
 * TestXmlPathDiscoveryGetPath.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.Web.UI;
using NUnit.Framework;
using NUnit.Extensions.Asp;
using NUnit.Extensions.Asp.AspTester;
using TopCoder.Web.UI.WebControl.BreadCrumb.Tests;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="XmlPathDiscovery"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// Since GetPath requires a Session instance, an IIS server is used.
    /// The aspx file TestXmlPathDiscovery.aspx is used to create the instance,
    /// call GetPath, and render the result in a grid.
    /// The sitemap test_files\BreadCrumbTrailWeb\XmlPathDiscovery.xml is used for the test.
    /// </remarks>
    [TestFixture]
    public class TestXmlPathDiscoveryGetPath : WebFormTestCase
    {
        /// <summary>
        /// The path of the aspx file.
        /// </summary>
        private const string TestPath = TestCommon.AppPath + "/TestXmlPathDiscovery.aspx";

        /// <summary>
        /// The url of the aspx used to test XmlPathDiscovery.
        /// </summary>
        private const string TestUrl = TestCommon.IisServer + TestPath;

        /// <summary>
        /// Represents the tbTitle text box. Initialized in SetUp.
        /// </summary>
        private TextBoxTester tbTitle;

        /// <summary>
        /// Represents the tbUrl text box. Initialized in SetUp.
        /// </summary>
        private TextBoxTester tbUrl;

        /// <summary>
        /// Represents the btnGetPath button. Initialized in SetUp.
        /// </summary>
        private ButtonTester btnGetPath;

        /// <summary>
        /// Represents the lblCount label. Initialized in SetUp.
        /// </summary>
        private LabelTester lblCount;

        /// <summary>
        /// Represents the gridPath DataGrid. Initialized in SetUp.
        /// </summary>
        private DataGridTester gridPath;

        /// <summary>
        /// Initializes the control testers.
        /// </summary>
        protected override void SetUp()
        {
            tbTitle = new TextBoxTester("tbTitle", CurrentWebForm);
            tbUrl = new TextBoxTester("tbUrl", CurrentWebForm);
            btnGetPath = new ButtonTester("btnGetPath", CurrentWebForm);
            lblCount = new LabelTester("lblCount", CurrentWebForm);
            gridPath = new DataGridTester("gridPath", CurrentWebForm);

            Browser.GetPage(TestUrl);
        }

        /// <summary>
        /// Tests GetPath for the three defined root nodes.
        /// Expecting only a single node in the returned path.
        /// </summary>
        [Test]
        public void TestRootNodes()
        {
            // R1 url
            tbTitle.Text = "First root";
            tbUrl.Text = "/R1";
            btnGetPath.Click();
            AssertEquals("R1 root node", "1", lblCount.Text);
            AssertEquals("R1 path", new string[][]
                {new string[] {"First root", "/R1"}}, gridPath.TrimmedCells);

            // R2 url (with query string)
            tbTitle.Text = "Second root";
            tbUrl.Text = "/R2?param=val";
            btnGetPath.Click();
            AssertEquals("R2 root node", "1", lblCount.Text);
            AssertEquals("R2 path", new string[][]
                {new string[] {"Second root", "/R2?param=val"}}, gridPath.TrimmedCells);

            // R3 url (single root without any children)
            tbTitle.Text = "Single root";
            tbUrl.Text = "/R3";
            btnGetPath.Click();
            AssertEquals("R3 single root node", "1", lblCount.Text);
            AssertEquals("R3 path", new string[][]
                {new string[] {"Single root", "/R3"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for level two nodes that each has a single path to root.
        /// There are three such nodes defined.
        /// Expecting only two nodes in each returned path.
        /// </summary>
        [Test]
        public void TestLevelTwoNodes()
        {
            // N11 url
            tbTitle.Text = "N11 node";
            tbUrl.Text = "/R1/N11";
            btnGetPath.Click();
            AssertEquals("N11 is below first root", "2", lblCount.Text);
            AssertEquals("N11 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N11 node", "/R1/N11"}}, gridPath.TrimmedCells);

            // N20 url (on first root with optional query string)
            tbTitle.Text = "N20 node";
            tbUrl.Text = "/N20";
            btnGetPath.Click();
            AssertEquals("N20 is below first root", "2", lblCount.Text);
            AssertEquals("N20 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N20 node", "/N20"}}, gridPath.TrimmedCells);

            // N22 url (on second root)
            tbTitle.Text = "N22 node";
            tbUrl.Text = "/R2/N22";
            btnGetPath.Click();
            AssertEquals("N22 is below second root", "2", lblCount.Text);
            AssertEquals("N22 path", new string[][]
                {new string[] {"R2", "/R2"},
                 new string[] {"N22 node", "/R2/N22"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for level three nodes that each has a single path to root.
        /// There is one such node defined.
        /// Expecting three nodes in each returned path.
        /// </summary>
        [Test]
        public void TestLevelThreeNodes()
        {
            // L1 url (with query string)
            tbTitle.Text = "L1 query";
            tbUrl.Text = "/L1?query=str";
            btnGetPath.Click();

            AssertEquals("L1 is two levels below first root", "3", lblCount.Text);
            AssertEquals("L1 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N11", "/R1/N11"},
                 new string[] {"L1 query", "/L1?query=str"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for level three nodes that has multiple path to root.
        /// There are two such nodes defined.
        /// Expecting the shortest path in each returned path.
        /// </summary>
        [Test]
        public void TestLevelThreeNodesMultiplePath()
        {
            // N12 has 2 path to root: N12->R1 and N12->N22->R2
            tbTitle.Text = "N12 shortest";
            tbUrl.Text = "/R1/N12";
            btnGetPath.Click();

            AssertEquals("N12 shortest", "2", lblCount.Text);
            AssertEquals("N12 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N12 shortest", "/R1/N12"}}, gridPath.TrimmedCells);

            // L2 has 2 path to root: L2->N20->R1 and L2->R2
            tbTitle.Text = "L2 shortest";
            tbUrl.Text = "/L2?a=b";
            btnGetPath.Click();

            AssertEquals("L2 shortest", "2", lblCount.Text);
            AssertEquals("L2 path", new string[][]
                {new string[] {"R2", "/R2"},
                 new string[] {"L2 shortest", "/L2?a=b"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for node that has no parent (loop).
        /// Expecting null is returned.
        /// </summary>
        [Test]
        public void TestLoop()
        {
            // Loop1
            tbTitle.Text = "First loop";
            tbUrl.Text = "/loop1";
            btnGetPath.Click();

            AssertEquals("Loop1 has no parent", "-1", lblCount.Text);

            // Loop2 (with query string)
            tbTitle.Text = "Second loop";
            tbUrl.Text = "/loop2?lo=op";
            btnGetPath.Click();

            AssertEquals("Loop2 has no parent", "-1", lblCount.Text);
        }

        /// <summary>
        /// Tests GetPath for node that doesn't exist in the map.
        /// Expecting null is returned.
        /// </summary>
        [Test]
        public void TestNotExist()
        {
            tbTitle.Text = "Not exist";
            tbUrl.Text = "/NotExist";
            btnGetPath.Click();

            AssertEquals("No path found", "-1", lblCount.Text);
        }

        /// <summary>
        /// Tests GetPath given null/uninitialized node (Url is null).
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathNullNode()
        {
            new XmlPathDiscovery(@"..\..\test_files\TextXmlPathDiscovery.xml").GetPath(
                new BreadCrumbNode(), new Page());
        }

        /// <summary>
        /// Tests GetPath given null page.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathNullPage()
        {
            new XmlPathDiscovery(@"..\..\test_files\TextXmlPathDiscovery.xml").GetPath(
                new BreadCrumbNode("title", "url"), null);
        }

        /// <summary>
        /// Tests GetPath given page that has no Session.
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetPathPageNoSession()
        {
            new XmlPathDiscovery(@"..\..\test_files\TextXmlPathDiscovery.xml").GetPath(
                new BreadCrumbNode("title", "url"), new Page());
        }
    }
}

