/*
 * TestOrderedListPathWriter.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Web.UI;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="OrderedListPathWriter"/> class via <see cref="MyOrderedListPathWriter"/>.
    /// </summary>
    ///
    /// <remarks>
    /// The protected methods are exposed to public in <see cref="MyOrderedListPathWriter"/>
    /// so that testing can be concentrated on each method.
    /// </remarks>
    [TestFixture]
    public class TestOrderedListPathWriter
    {
        /// <summary>
        /// The OrderedListPathWriter to test. Initialized in SetUp.
        /// </summary>
        private MyOrderedListPathWriter writer;

        /// <summary>
        /// A BreadCrumbTrailControl containing settings to use. Initialized in SetUp.
        /// </summary>
        private BreadCrumbTrailControl control;

        /// <summary>
        /// A sample collection of BreadCrumbNode. Initialized in SetUp.
        /// </summary>
        private BreadCrumbNode[] nodes;

        /// <summary>
        /// The HtmlTextWriter to use. Initialized in SetUp.
        /// </summary>
        private HtmlTextWriter htmlWriter;

        /// <summary>
        /// The StringWriter which is used by the htmlWriter to write to. Initialized in SetUp.
        /// </summary>
        private StringWriter strWriter;

        /// <summary>
        /// Initializes the BreadCrumbTrailControl and BreadCrumbNode for testing.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // init control with some initial settings
            control = new BreadCrumbTrailControl();
            control.PathSeparator = "PS";
            control.NodeStyle = "NS";
            control.CurrentNodeStyle = "CNS";
            control.RootNodeStyle = "RNS";
            control.MouseOverStyle = "MOS";
            control.PathDirection = PathDirection.RootToCurrent;
            control.ErrorDisposition = ErrorDisposition.RenderException;

            // prepare 3 sample nodes: [0] is root, [2] is current
            nodes = new BreadCrumbNode[3];
            nodes[0] = new BreadCrumbNode("Node0", "Url0");
            nodes[1] = new BreadCrumbNode("Node1", "Url1");
            nodes[2] = new BreadCrumbNode("Node2", "Url2");

            // initialized writers
            strWriter = new StringWriter();
            htmlWriter = new HtmlTextWriter(strWriter, "");  // no indent, no newline
            htmlWriter.NewLine = "";

            writer = new MyOrderedListPathWriter();
        }

        /// <summary>
        /// Tests BeginPath accuracy to write an opening OL tag.
        /// </summary>
        [Test]
        public void TestBeginPath()
        {
            writer.MyBeginPath(htmlWriter);

            // assert opening OL tag
            string actual = strWriter.ToString();
            Assert.AreEqual("<ol>",
                actual, "BeginPath result is incorrect");
        }

        /// <summary>
        /// Tests BeginPath given null writer.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestBeginPathNull()
        {
            writer.MyBeginPath(null);
        }

        /// <summary>
        /// Tests WriteNode accuracy to write a single node with all corresponding style
        /// which is encapsulated as a list item (LI tag).
        /// </summary>
        [Test]
        public void TestWriteNode()
        {
            writer.MyWriteNode(htmlWriter, nodes[0], "NS", "MOS");

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a class=\"NS\" onMouseOver=\"this.className='MOS'\" onMouseOut=\"this.className='NS'\" "
                + "href=\"Url0\">Node0</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode accuracy given null nodeStyle.
        /// Class and OnMouseOut attributes should not be rendered.
        /// </summary>
        [Test]
        public void TestWriteNodeNoNodeStyle()
        {
            writer.MyWriteNode(htmlWriter, nodes[0], null, "MOS");

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a onMouseOver=\"this.className='MOS'\" onMouseOut=\"this.className=''\" "
                + "href=\"Url0\">Node0</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode accuracy given empty nodeStyle.
        /// Similar to null, Class and OnMouseOut attributes should not be rendered.
        /// </summary>
        [Test]
        public void TestWriteNodeEmptyNodeStyle()
        {
            writer.MyWriteNode(htmlWriter, nodes[0], " ", "MOS");

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a onMouseOver=\"this.className='MOS'\" onMouseOut=\"this.className=''\" "
                + "href=\"Url0\">Node0</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode accuracy given null mouseOverStyle.
        /// OnMouseXXX attributes should not be rendered.
        /// </summary>
        [Test]
        public void TestWriteNodeNoMouseOverStyle()
        {
            writer.MyWriteNode(htmlWriter, nodes[0], "NS", null);

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a class=\"NS\" href=\"Url0\">Node0</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode accuracy given empty mouseOverStyle.
        /// Similar to null, OnMouseXXX attributes should not be rendered.
        /// </summary>
        [Test]
        public void TestWriteNodeEmptyMouseOverStyle()
        {
            writer.MyWriteNode(htmlWriter, nodes[0], "NS", " ");

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a class=\"NS\" href=\"Url0\">Node0</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode renders the title html encoded.
        /// The Url part should be rendered as is.
        /// </summary>
        [Test]
        public void TestWriteNodeHtmlEncode()
        {
            nodes[0] = new BreadCrumbNode("<Node0>", "http://test/page?par=<val? =>");
            writer.MyWriteNode(htmlWriter, nodes[0], null, null);

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a href=\"http://test/page?par=<val? =>\">&lt;Node0&gt;</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode renders a default title ("Unknown") given a node with a null title.
        /// </summary>
        [Test]
        public void TestWriteNodeNullTitle()
        {
            nodes[0] = new BreadCrumbNode(null, "http://url");
            writer.MyWriteNode(htmlWriter, nodes[0], null, null);

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a href=\"http://url\">Unknown</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode renders a default title ("Unknown") given a node with empty title.
        /// </summary>
        [Test]
        public void TestWriteNodeEmptyTitle()
        {
            nodes[0] = new BreadCrumbNode(" ", "http://empty");
            writer.MyWriteNode(htmlWriter, nodes[0], null, null);

            // assert node is rendered correctly
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<li><a href=\"http://empty\">Unknown</a></li>",

                actual, "WriteNode result is incorrect");
        }

        /// <summary>
        /// Tests WriteNode given null writer.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeNullWriter()
        {
            writer.MyWriteNode(null, nodes[0], null, null);
        }

        /// <summary>
        /// Tests WriteNode given null/uninitialized node url.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeNullNode()
        {
            writer.MyWriteNode(htmlWriter, new BreadCrumbNode(), null, null);
        }

        /// <summary>
        /// Tests using WritePath to write 3 nodes in RootToCurrent order.
        /// </summary>
        [Test]
        public void TestWritePathRootToCurrent()
        {
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.IsNull(ex, "No exception occurs");

            // assert all 3 nodes are written correctly as listed order
            string actual = strWriter.ToString();
            Assert.AreEqual(
                "<ol>"

                // first (root) node
                + "<li><a class=\"RNS\" onMouseOver=\"this.className='MOS'\" onMouseOut=\"this.className='RNS'\" "
                + "href=\"Url0\">Node0</a></li>"

                // 2nd node
                + "<li><a class=\"NS\" onMouseOver=\"this.className='MOS'\" onMouseOut=\"this.className='NS'\" "
                + "href=\"Url1\">Node1</a></li>"

                // last (current) node
                + "<li><a class=\"CNS\" onMouseOver=\"this.className='MOS'\" onMouseOut=\"this.className='CNS'\" "
                + "href=\"Url2\">Node2</a></li>"

                + "</ol>",

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WriteSeparator really writes nothing.
        /// </summary>
        [Test]
        public void TestWriteSeparator()
        {
            writer.MyWriteSeparator(htmlWriter, control);

            // assert nothing is written
            string actual = strWriter.ToString();
            Assert.AreEqual("", actual, "WriteSeparator should not write anything");
        }

        /// <summary>
        /// Tests WriteSeparator given null writer and null parent control.
        /// Expecting no exception is thrown.
        /// </summary>
        [Test]
        public void TestWriteSeparatorNull()
        {
            writer.MyWriteSeparator(null, null);
        }

        /// <summary>
        /// Tests EndPath accuracy to write a closing OL tag.
        /// </summary>
        [Test]
        public void TestEndPath()
        {
            writer.MyBeginPath(htmlWriter);
            writer.MyEndPath(htmlWriter);

            // assert opening and closing OL tag
            string actual = strWriter.ToString();
            Assert.AreEqual("<ol></ol>",
                actual, "EndPath result is incorrect");
        }

        /// <summary>
        /// Tests EndPath given null writer.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestEndPathNull()
        {
            writer.MyEndPath(null);
        }

        /// <summary>
        /// Tests EndPath without invoking BeginPath.
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestEndPathWithoutBeginPath()
        {
            writer.MyEndPath(htmlWriter);
        }
    }
}
