/*
 * TestAbstractPathWriter.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Web.UI;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="AbstractPathWriter"/> class via the concrete class
    /// <see cref="MyPathWriter"/>.
    /// </summary>
    ///
    /// <remarks>
    /// <see cref="MyPathWriter"/> provides simple implementation of the various protected methods
    /// which writes some dummy string into the html writer.
    /// The resulting output will then be asserted to ensure that methods are invoked correctly
    /// (in the correct order and that the correct node is passed).
    /// </remarks>
    [TestFixture]
    public class TestAbstractPathWriter
    {
        /// <summary>
        /// The AbstractPathWriter to test. Initialized in SetUp.
        /// </summary>
        private IPathWriter writer;

        /// <summary>
        /// A BreadCrumbTrailControl containing settings to use. Initialized in SetUp.
        /// </summary>
        private BreadCrumbTrailControl control;

        /// <summary>
        /// A sample collection of BreadCrumbNode. Initialized in SetUp.
        /// </summary>
        private BreadCrumbNode[] nodes;

        /// <summary>
        /// The HtmlTextWriter to use. Initialized in SetUp.
        /// </summary>
        private HtmlTextWriter htmlWriter;

        /// <summary>
        /// The StringWriter which is used by the htmlWriter to write to. Initialized in SetUp.
        /// </summary>
        private StringWriter strWriter;

        /// <summary>
        /// Default PathSeparator html output result.
        /// {0} and {1} is placeholder for the style and the string itself.
        /// as initialized in SetUp.
        /// </summary>
        private const string PathSeparatorHtml = "<span class=\"{0}\">{1}</span>";

        /// <summary>
        /// Initializes the BreadCrumbTrailControl and BreadCrumbNode for testing.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // init control with some initial settings
            control = new BreadCrumbTrailControl();
            control.PathSeparator = "PS";
            control.PathSeparatorStyle = "PSS";
            control.NodeStyle = "NS";
            control.CurrentNodeStyle = "CNS";
            control.RootNodeStyle = "RNS";
            control.MouseOverStyle = "MOS";
            control.PathDirection = PathDirection.RootToCurrent;
            control.ErrorDisposition = ErrorDisposition.RenderException;

            // prepare 3 sample nodes: [0] is root, [2] is current
            nodes = new BreadCrumbNode[3];
            nodes[0] = new BreadCrumbNode("Node0", "Url0");
            nodes[1] = new BreadCrumbNode("Node1", "Url1");
            nodes[2] = new BreadCrumbNode("Node2", "Url2");

            // initialized writers
            strWriter = new StringWriter();
            htmlWriter = new HtmlTextWriter(strWriter, "");  // no indent, no newline
            htmlWriter.NewLine = "";

            writer = new MyPathWriter();
        }

        /// <summary>
        /// Tests WritePath accuracy to write 3 nodes in RootToCurrent order.
        /// </summary>
        [Test]
        public void TestWritePathRootToCurrent()
        {
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.IsNull(ex, "No exception occurs");

            // assert all 3 nodes are written correctly, along with its style
            // and that beginpath/endpath is called in the correct order
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath accuracy to write 3 nodes in CurrentToRoot order.
        /// </summary>
        [Test]
        public void TestWritePathCurrentToRoot()
        {
            control.PathDirection = PathDirection.CurrentToRoot;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.IsNull(ex, "No exception occurs");

            // assert all 3 nodes are written correctly, along with its style
            // and that beginpath/endpath is called in the correct order
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // last (current) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // first (root) node
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath accuracy to write only 1 node.
        /// </summary>
        [Test]
        public void TestWritePathSingleNode()
        {
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, new BreadCrumbNode[] {nodes[0]}, control);
            Assert.IsNull(ex, "No exception occurs");

            // assert that the single node is rendered in RootNodeStyle
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // node rendered in CurrentNodeStyle
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath accuracy to write 2 nodes and no PathSeparator defined.
        /// </summary>
        [Test]
        public void TestWritePathNoPathSeparator()
        {
            control.PathSeparator = null;
            BreadCrumbException[] ex = writer.WritePath(
                htmlWriter, new BreadCrumbNode[] {nodes[0], nodes[1]}, control);
            Assert.IsNull(ex, "No exception occurs");

            // assert all 2 nodes are written correctly without any separator
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node1", "CNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath accuracy to write a single node and no PathSeparatorStyle defined.
        /// Also assert that the PathSeparator is HtmlEncoded when necessary.
        /// </summary>
        [Test]
        public void TestWritePathNoPathSeparatorStyle()
        {
            control.PathSeparatorStyle = null;
            control.PathSeparator = "<PS>";  // need to be html encoded
            control.PathDirection = PathDirection.CurrentToRoot; // use CurrentToRoot
            BreadCrumbException[] ex = writer.WritePath(
                htmlWriter, new BreadCrumbNode[] {nodes[1], nodes[2]}, control);
            Assert.IsNull(ex, "No exception occurs");

            // assert node is written correctly without any separator style
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (current) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS")
                + "&lt;PS&gt;"

                // last (root) node
                + string.Format(MyPathWriter.NodeHtml, "Node1", "RNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath accuracy given a control that has null reference
        /// for both CurrentNodeStyle and RootNodeStyle.
        /// Expecting that NodeStyle is used instead.
        /// </summary>
        [Test]
        public void TestWritePathUseNodeStyle()
        {
            control.CurrentNodeStyle = null;
            control.RootNodeStyle = null;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.IsNull(ex, "No exception occurs");

            // assert node is written correctly using NS for all styles
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node0", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "NS", "MOS")
                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given null writer.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWritePathNullWriter()
        {
            writer.WritePath(null, nodes, control);
        }

        /// <summary>
        /// Tests WritePath given null nodes.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWritePathNullNodes()
        {
            writer.WritePath(htmlWriter, null, control);
        }

        /// <summary>
        /// Tests WritePath given null parent control.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWritePathNullParentControl()
        {
            writer.WritePath(htmlWriter, nodes, null);
        }

        /// <summary>
        /// Tests WritePath given empty array nodes.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestWritePathEmptyNodes()
        {
            writer.WritePath(htmlWriter, new BreadCrumbNode[0], control);
        }

        /// <summary>
        /// Tests WritePath given nodes with uninitialized element.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestWritePathUninitializedNodes()
        {
            writer.WritePath(htmlWriter, new BreadCrumbNode[2], control);
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at BeginPath.
        /// The ErrorDisposition is RenderException (as initialized in SetUp).
        /// Processing continues, nodes are rendered, and the method returns 1 exception.
        /// </summary>
        [Test]
        public void TestWritePathBeginPathRenderException()
        {
            writer = new MyPathWriter(MyPathWriter.BeginPathHtml);
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.AreEqual(1, ex.Length, "Exception at BeginPath");
            Assert.IsTrue(ex[0].InnerException is InvalidOperationException, "Actual inner exception");

            // assert all 3 nodes are still rendered (but no BeginPath)
            string actual = strWriter.ToString();
            Assert.AreEqual(
                // first (root) node + separator
                string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at BeginPath.
        /// The ErrorDisposition is SilentlyIgnore.
        /// Processing continues, nodes are rendered, and the method returns no exception.
        /// </summary>
        [Test]
        public void TestWritePathBeginPathSilentlyIgnore()
        {
            writer = new MyPathWriter(MyPathWriter.BeginPathHtml);
            control.ErrorDisposition = ErrorDisposition.SilentlyIgnore;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.IsNull(ex, "Exception is ignored");

            // assert all 3 nodes are still rendered (but no BeginPath)
            string actual = strWriter.ToString();
            Assert.AreEqual(
                // first (root) node + separator
                string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at BeginPath.
        /// The ErrorDisposition is ThrowException.
        /// Processing is aborted, no node is rendered, and the method returns 1 exception.
        /// </summary>
        [Test]
        public void TestWritePathBeginPathThrowException()
        {
            writer = new MyPathWriter(MyPathWriter.BeginPathHtml);
            control.ErrorDisposition = ErrorDisposition.ThrowException;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.AreEqual(1, ex.Length, "Exception at BeginPath");
            Assert.IsTrue(ex[0].InnerException is InvalidOperationException, "Actual inner exception");

            // assert no node is rendered
            string actual = strWriter.ToString();
            Assert.AreEqual("", actual, "Process is aborted");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at EndPath.
        /// The ErrorDisposition is RenderException (as initialized in SetUp).
        /// Processing continues, nodes are rendered, and the method returns 1 exception.
        /// </summary>
        [Test]
        public void TestWritePathEndPathRenderException()
        {
            writer = new MyPathWriter(MyPathWriter.EndPathHtml);
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.AreEqual(1, ex.Length, "Exception at EndPath");
            Assert.IsTrue(ex[0].InnerException is InvalidOperationException, "Actual inner exception");

            // assert all 3 nodes are still rendered (but no EndPath)
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS"),

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at EndPath.
        /// The ErrorDisposition is SilentlyIgnore.
        /// Processing continues, nodes are rendered, and the method returns no exception.
        /// </summary>
        [Test]
        public void TestWritePathEndPathSilentlyIgnore()
        {
            writer = new MyPathWriter(MyPathWriter.EndPathHtml);
            control.ErrorDisposition = ErrorDisposition.SilentlyIgnore;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.IsNull(ex, "Exception is ignored");

            // assert all 3 nodes are still rendered (but no EndPath)
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS"),

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at EndPath.
        /// The ErrorDisposition is ThrowException.
        /// Processing is aborted, however nodes are rendered already,
        /// and the method returns 1 exception.
        /// </summary>
        [Test]
        public void TestWritePathEndPathThrowException()
        {
            writer = new MyPathWriter(MyPathWriter.EndPathHtml);
            control.ErrorDisposition = ErrorDisposition.ThrowException;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.AreEqual(1, ex.Length, "Exception at EndPath");
            Assert.IsTrue(ex[0].InnerException is InvalidOperationException, "Actual inner exception");

            // assert all 3 nodes are already rendered (but no EndPath)
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // 2nd node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS"),

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at WriteNode.
        /// The ErrorDisposition is RenderException (as initialized in SetUp).
        /// Processing continues, all nodes (except the erroneous node) are rendered,
        /// and the method returns 1 exception.
        /// </summary>
        [Test]
        public void TestWritePathWriteNodeRenderException()
        {
            // fail at second node
            writer = new MyPathWriter("Node1");
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.AreEqual(1, ex.Length, "Exception at WriteNode");
            Assert.IsTrue(ex[0].InnerException is InvalidOperationException, "Actual inner exception");

            // assert 2 nodes are rendered (but no Node1)
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // last (current) node
                + string.Format(MyPathWriter.NodeHtml, "Node2", "CNS", "MOS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at WriteNode.
        /// The ErrorDisposition is SilentlyIgnore.
        /// Processing continues, all nodes (except the erroneous node) are rendered,
        /// and the method returns no exception.
        /// </summary>
        [Test]
        public void TestWritePathWriteNodeSilentlyIgnore()
        {
            // fail on last/current node
            writer = new MyPathWriter("Node2");
            control.ErrorDisposition = ErrorDisposition.SilentlyIgnore;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.IsNull(ex, "Exception is ignored");

            // assert 2 nodes are rendered (but no Node2)
            string actual = strWriter.ToString();
            Assert.AreEqual(
                MyPathWriter.BeginPathHtml

                // first (root) node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node0", "RNS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                // second node + separator
                + string.Format(MyPathWriter.NodeHtml, "Node1", "NS", "MOS")
                + string.Format(PathSeparatorHtml, "PSS", "PS")

                + MyPathWriter.EndPathHtml,

                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests WritePath given a writer that throws exception at WriteNode.
        /// The ErrorDisposition is ThrowException.
        /// Processing is aborted, next nodes are not rendered,
        /// and the method returns 1 exception.
        /// </summary>
        [Test]
        public void TestWritePathWriteNodeThrowException()
        {
            // fail on 1st node
            writer = new MyPathWriter("Node0");
            control.ErrorDisposition = ErrorDisposition.ThrowException;
            BreadCrumbException[] ex = writer.WritePath(htmlWriter, nodes, control);
            Assert.AreEqual(1, ex.Length, "Exception at WriteNode");
            Assert.IsTrue(ex[0].InnerException is InvalidOperationException, "Actual inner exception");

            // assert no nodes are rendered (because it fails on first node and aborted)
            string actual = strWriter.ToString();
            Assert.AreEqual(MyPathWriter.BeginPathHtml,
                actual, "WritePath result is incorrect");
        }

        /// <summary>
        /// Tests BeginPath given null writer. It is tested via MyBeginPath.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestBeginPathNull()
        {
            ((MyPathWriter) writer).MyBeginPath(null);
        }

        /// <summary>
        /// Tests EndPath given null writer. It is tested via MyEndPath.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestEndPathNull()
        {
            ((MyPathWriter) writer).MyEndPath(null);
        }

        /// <summary>
        /// Tests WriteSeparator given null writer. It is tested via MyWriteSeparator.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteSeparatorNullWriter()
        {
            ((MyPathWriter) writer).MyWriteSeparator(null, control);
        }

        /// <summary>
        /// Tests WriteSeparator given null parent control. It is tested via MyWriteSeparator.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteSeparatorNullParent()
        {
            ((MyPathWriter) writer).MyWriteSeparator(htmlWriter, null);
        }
    }
}
