/*
 * TestBreadCrumbTrailControl.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Data;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="BreadCrumbTrailControl"/> class.
    /// </summary>
    [TestFixture]
    public class TestBreadCrumbTrailControl
    {
        /// <summary>
        /// The control to test. Initialized in SetUp.
        /// </summary>
        private BreadCrumbTrailControl control;

        /// <summary>
        /// Initializes the <see cref="BreadCrumbTrailControl"/> to test.
        /// No configuration is given, so it should initialize all properties to default values.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // clean up config manager namespace and instantiate instance
            TestCommon.LoadConfigFile();
            control = new BreadCrumbTrailControl();
        }

        /// <summary>
        /// Cleans up config manager namespace.
        /// </summary>
        [TearDown]
        public void CleanUpCm()
        {
            TestCommon.LoadConfigFile();
        }

        /// <summary>
        /// Tests PathSeparator property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestPathSeparator()
        {
            // default value
            Assert.AreEqual(":", control.PathSeparator, "Default value");

            // try setting and reading back
            control.PathSeparator = "<<A-Z>>";
            Assert.AreEqual("<<A-Z>>", control.PathSeparator, "Setter and getter");

            // set to empty string (empty is accepted)
            control.PathSeparator = " ";
            Assert.AreEqual(" ", control.PathSeparator, "Setter and getter empty");

            // set to null
            control.PathSeparator = null;
            Assert.IsNull(control.PathSeparator, "Setter and getter null");
        }

        /// <summary>
        /// Tests PathSeparatorStyle property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestPathSeparatorStyle()
        {
            // default value
            Assert.IsNull(control.PathSeparatorStyle, "Default value");

            // try setting and reading back
            control.PathSeparatorStyle = "-PSS- ";
            Assert.AreEqual("-PSS- ", control.PathSeparatorStyle, "Setter and getter");

            // set to empty string (converted to null)
            control.PathSeparatorStyle = " ";
            Assert.IsNull(control.PathSeparatorStyle, "Setter and getter empty");

            // set to null
            control.PathSeparatorStyle = null;
            Assert.IsNull(control.PathSeparatorStyle, "Setter and getter null");
        }

        /// <summary>
        /// Tests NodeStyle property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestNodeStyle()
        {
            // default value
            Assert.IsNull(control.NodeStyle, "Default value");

            // try setting and reading back
            control.NodeStyle = " -NS-";
            Assert.AreEqual(" -NS-", control.NodeStyle, "Setter and getter");

            // set to empty string (converted to null)
            control.NodeStyle = " ";
            Assert.IsNull(control.NodeStyle, "Setter and getter empty");

            // set to null
            control.NodeStyle = null;
            Assert.IsNull(control.NodeStyle, "Setter and getter null");
        }

        /// <summary>
        /// Tests CurrentNodeStyle property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestCurrentNodeStyle()
        {
            // default value
            Assert.IsNull(control.CurrentNodeStyle, "Default value");

            // try setting and reading back
            control.CurrentNodeStyle = "CSS";
            Assert.AreEqual("CSS", control.CurrentNodeStyle, "Setter and getter");

            // set to empty string (converted to null)
            control.CurrentNodeStyle = " ";
            Assert.IsNull(control.CurrentNodeStyle, "Setter and getter empty");

            // set to null
            control.CurrentNodeStyle = null;
            Assert.IsNull(control.CurrentNodeStyle, "Setter and getter null");
        }

        /// <summary>
        /// Tests RootNodeStyle property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestRootNodeStyle()
        {
            // default value
            Assert.IsNull(control.RootNodeStyle, "Default value");

            // try setting and reading back
            control.RootNodeStyle = "RNS";
            Assert.AreEqual("RNS", control.RootNodeStyle, "Setter and getter");

            // set to empty string (converted to null)
            control.RootNodeStyle = " ";
            Assert.IsNull(control.RootNodeStyle, "Setter and getter empty");

            // set to null
            control.RootNodeStyle = null;
            Assert.IsNull(control.RootNodeStyle, "Setter and getter null");
        }

        /// <summary>
        /// Tests MouseOverStyle property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestMouseOverStyle()
        {
            // default value
            Assert.IsNull(control.MouseOverStyle, "Default value");

            // try setting and reading back
            control.MouseOverStyle = "MOS";
            Assert.AreEqual("MOS", control.MouseOverStyle, "Setter and getter");

            // set to empty string (converted to null)
            control.MouseOverStyle = " ";
            Assert.IsNull(control.MouseOverStyle, "Setter and getter empty");

            // set to null
            control.MouseOverStyle = null;
            Assert.IsNull(control.MouseOverStyle, "Setter and getter null");
        }

        /// <summary>
        /// Tests ErrorDisposition property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, integer, and out-of-range value.
        /// </summary>
        [Test]
        public void TestErrorDisposition()
        {
            // default value
            Assert.AreEqual(ErrorDisposition.SilentlyIgnore, control.ErrorDisposition, "Default value");

            // try setting and reading back
            control.ErrorDisposition = ErrorDisposition.RenderException;
            Assert.AreEqual(ErrorDisposition.RenderException, control.ErrorDisposition, "Setter and getter");

            // try setting using integer value
            control.ErrorDisposition = (ErrorDisposition) 2;
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition, "Setter and getter int");

            // set to out-of-range value (ArgumentException expected)
            try
            {
                control.ErrorDisposition = (ErrorDisposition) 100;
                Assert.Fail("Expecting ArgumentException");
            }
            catch (ArgumentException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests PathDirection property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, integer, and out-of-range value.
        /// </summary>
        [Test]
        public void TestPathDirection()
        {
            // default value
            Assert.AreEqual(PathDirection.RootToCurrent, control.PathDirection, "Default value");

            // try setting and reading back
            control.PathDirection = PathDirection.CurrentToRoot;
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection, "Setter and getter");

            // try setting using integer value
            control.PathDirection = (PathDirection) 0;
            Assert.AreEqual(PathDirection.RootToCurrent, control.PathDirection, "Setter and getter int");

            // set to out-of-range value (ArgumentException expected)
            try
            {
                control.PathDirection = (PathDirection) 100;
                Assert.Fail("Expecting ArgumentException");
            }
            catch (ArgumentException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests DataSource property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using IList.
        /// </summary>
        [Test]
        public void TestDataSourceIList()
        {
            // default value
            Assert.IsNull(control.DataSource, "Default value");

            // try setting and reading back using IList
            ArrayList newData = new ArrayList();
            newData.Add(new BreadCrumbNode("a", "b"));
            control.DataSource = newData;
            Assert.AreSame(newData, control.DataSource, "Setter and getter IList");

            // set to null
            control.DataSource = null;
            Assert.IsNull(control.DataSource, "Setter and getter null");
        }

        /// <summary>
        /// Tests DataSource property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using BreadCrumbNode[].
        /// </summary>
        [Test]
        public void TestDataSourceArray()
        {
            // default value
            Assert.IsNull(control.DataSource, "Default value");

            // try setting and reading back using BreadCrumbNode[]
            BreadCrumbNode[] nodes = new BreadCrumbNode[] { new BreadCrumbNode("a", "b") };
            control.DataSource = nodes;
            Assert.AreEqual(1, ((BreadCrumbNode[]) control.DataSource).Length, "Setter and getter array length");
            Assert.AreEqual("a", ((BreadCrumbNode[]) control.DataSource)[0].Title, "Setter and getter array Title");
            Assert.AreEqual("b", ((BreadCrumbNode[]) control.DataSource)[0].Url, "Setter and getter array Url");

            // set to null
            control.DataSource = null;
            Assert.IsNull(control.DataSource, "Setter and getter null");
        }

        /// <summary>
        /// Tests DataSource property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using DataTable.
        /// </summary>
        [Test]
        public void TestDataSourceDataTable()
        {
            // default value
            Assert.IsNull(control.DataSource, "Default value");

            // try setting and reading back using DataTable
            DataTable tbl = new DataTable();
            tbl.Columns.Add("Title");
            tbl.Columns.Add("Url");
            control.DataSource = tbl;
            Assert.AreSame(tbl, control.DataSource, "Setter and getter DataTable");

            // set to null
            control.DataSource = null;
            Assert.IsNull(control.DataSource, "Setter and getter null");
        }

        /// <summary>
        /// Tests DataSource property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using DataSet.
        /// </summary>
        [Test]
        public void TestDataSourceDataSet()
        {
            // default value
            Assert.IsNull(control.DataSource, "Default value");

            // try setting and reading back using DataSet
            DataTable tbl = new DataTable();
            tbl.Columns.Add("Title");
            tbl.Columns.Add("Url");
            tbl.Columns.Add("AdditionalIsOk");
            DataSet ds = new DataSet();
            ds.Tables.Add(tbl);

            control.DataSource = ds;
            Assert.AreSame(ds, control.DataSource, "Setter and getter DataSet");

            // set to null
            control.DataSource = null;
            Assert.IsNull(control.DataSource, "Setter and getter null");
        }

        /// <summary>
        /// Tests DataSource given empty IList.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceEmptyIList()
        {
            control.DataSource = new ArrayList();
        }

        /// <summary>
        /// Tests DataSource given IList containing null element.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceNullIList()
        {
            IList list = new ArrayList();
            list.Add(new BreadCrumbNode("a", "b"));
            list.Add(null);
            control.DataSource = list;
        }

        /// <summary>
        /// Tests DataSource given IList containing non BreadCrumbNode element.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceNonBcIList()
        {
            IList list = new ArrayList();
            list.Add("a");
            control.DataSource = list;
        }

        /// <summary>
        /// Tests DataSource given IList containing uninitialized BreadCrumbNode element.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceUninitializedBcIList()
        {
            IList list = new ArrayList();
            list.Add(new BreadCrumbNode());
            control.DataSource = list;
        }

        /// <summary>
        /// Tests DataSource given empty BreadCrumbNode[].
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceEmptyArray()
        {
            control.DataSource = new BreadCrumbNode[0];
        }

        /// <summary>
        /// Tests DataSource given uninitialized BreadCrumbNode[].
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceUninitializedArray()
        {
            // 2 elements: first element is not initialized with proper ctor
            BreadCrumbNode[] nodes = new BreadCrumbNode[2];
            nodes[1] = new BreadCrumbNode("a", "b");
            control.DataSource = nodes;
        }

        /// <summary>
        /// Tests DataSource given DataTable without Title column.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceDataTableTitle()
        {
            DataTable tbl = new DataTable();
            tbl.Columns.Add("Url");
            control.DataSource = tbl;
        }

        /// <summary>
        /// Tests DataSource given DataTable without Url column.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceDataTableUrl()
        {
            DataTable tbl = new DataTable();
            tbl.Columns.Add("Title");
            control.DataSource = tbl;
        }

        /// <summary>
        /// Tests DataSource given empty DataSet.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceDataSetEmpty()
        {
            control.DataSource = new DataSet();
        }

        /// <summary>
        /// Tests DataSource given DataSet whose first DataTable has no Title column.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceDataSetTitle()
        {
            DataTable tbl = new DataTable();
            tbl.Columns.Add("Url");
            DataSet ds = new DataSet();
            ds.Tables.Add(tbl);
            control.DataSource = ds;
        }

        /// <summary>
        /// Tests DataSource given DataSet whose first DataTable has no Url column.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceDataSetUrl()
        {
            DataTable tbl = new DataTable();
            tbl.Columns.Add("Title");
            DataSet ds = new DataSet();
            ds.Tables.Add(tbl);
            control.DataSource = ds;
        }

        /// <summary>
        /// Tests DataSource given the unsupported type.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceUnsupported()
        {
            control.DataSource = "unknown";
        }

        /// <summary>
        /// Tests Title property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestTitle()
        {
            // default value
            Assert.IsNull(control.Title, "Default value");

            // try setting and reading back
            control.Title = "New Title ";
            Assert.AreEqual("New Title ", control.Title, "Setter and getter");

            // set to empty string (converted to null)
            control.Title = " ";
            Assert.IsNull(control.Title, "Setter and getter empty");

            // set to null
            control.Title = null;
            Assert.IsNull(control.Title, "Setter and getter null");
        }

        /// <summary>
        /// Tests NodeDiscoveryKey property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestNodeDiscoveryKey()
        {
            // default value
            Assert.IsNull(control.NodeDiscoveryKey, "Default value");

            // try setting and reading back
            control.NodeDiscoveryKey = "NDK";
            Assert.AreEqual("NDK", control.NodeDiscoveryKey, "Setter and getter");

            // set to empty string (converted to null)
            control.NodeDiscoveryKey = " ";
            Assert.IsNull(control.NodeDiscoveryKey, "Setter and getter empty");

            // set to null
            control.NodeDiscoveryKey = null;
            Assert.IsNull(control.NodeDiscoveryKey, "Setter and getter null");
        }

        /// <summary>
        /// Tests PathDiscoveryKey property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestPathDiscoveryKey()
        {
            // default value
            Assert.IsNull(control.PathDiscoveryKey, "Default value");

            // try setting and reading back
            control.PathDiscoveryKey = "PDK";
            Assert.AreEqual("PDK", control.PathDiscoveryKey, "Setter and getter");

            // set to empty string (converted to null)
            control.PathDiscoveryKey = " ";
            Assert.IsNull(control.PathDiscoveryKey, "Setter and getter empty");

            // set to null
            control.PathDiscoveryKey = null;
            Assert.IsNull(control.PathDiscoveryKey, "Setter and getter null");
        }

        /// <summary>
        /// Tests PathWriterKey property accuracy.
        /// Tests the default value, then try setting and getting back the value
        /// using some value, empty string, and null.
        /// </summary>
        [Test]
        public void TestPathWriterKey()
        {
            // default value
            Assert.IsNull(control.PathWriterKey, "Default value");

            // try setting and reading back
            control.PathWriterKey = "PWK";
            Assert.AreEqual("PWK", control.PathWriterKey, "Setter and getter");

            // set to empty string (converted to null)
            control.PathWriterKey = " ";
            Assert.IsNull(control.PathWriterKey, "Setter and getter empty");

            // set to null
            control.PathWriterKey = null;
            Assert.IsNull(control.PathWriterKey, "Setter and getter null");
        }

        /// <summary>
        /// Tests instantiating control with initial default value defined
        /// in configuration file.
        /// </summary>
        [Test]
        public void TestWithConfig()
        {
            TestCommon.LoadConfigFile(@"..\..\test_files\TestBctc\Valid.xml");
            control = new BreadCrumbTrailControl();

            // make sure all properties are initialized as specified in config
            Assert.AreEqual("ConfigPS", control.PathSeparator, "PathSeparator");
            Assert.AreEqual("ConfigPSS", control.PathSeparatorStyle, "PathSeparatorStyle");
            Assert.AreEqual("ConfigNS", control.NodeStyle, "NodeStyle");
            Assert.AreEqual("ConfigRNS", control.RootNodeStyle, "RootNodeStyle");
            Assert.AreEqual("ConfigCNS", control.CurrentNodeStyle, "CurrentNodeStyle");
            Assert.AreEqual("ConfigMOS", control.MouseOverStyle, "MouseOverStyle");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection, "PathDirection");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition, "ErrorDisposition");
            Assert.AreEqual("NDK", control.NodeDiscoveryKey, "NodeDiscoveryKey");
            Assert.AreEqual("PDK", control.PathDiscoveryKey, "PathDiscoveryKey");
            Assert.AreEqual("PWK", control.PathWriterKey, "PathWriterKey");
        }

        /// <summary>
        /// Tests instantiating control with enumeration defined as integer value in the configuration file.
        /// </summary>
        [Test]
        public void TestWithConfigEnumInt()
        {
            TestCommon.LoadConfigFile(@"..\..\test_files\TestBctc\EnumInt.xml");
            control = new BreadCrumbTrailControl();

            // make sure all properties are initialized to default
            // and enumerator is initialized as specified in config
            Assert.AreEqual(":", control.PathSeparator, "PathSeparator");
            Assert.IsNull(control.PathSeparatorStyle, "PathSeparatorStyle");
            Assert.IsNull(control.NodeStyle, "NodeStyle");
            Assert.IsNull(control.RootNodeStyle, "RootNodeStyle");
            Assert.IsNull(control.CurrentNodeStyle, "CurrentNodeStyle");
            Assert.IsNull(control.MouseOverStyle, "MouseOverStyle");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection, "PathDirection");
            Assert.AreEqual(ErrorDisposition.RenderException, control.ErrorDisposition, "ErrorDisposition");
            Assert.IsNull(control.NodeDiscoveryKey, "NodeDiscoveryKey");
            Assert.IsNull(control.PathDiscoveryKey, "PathDiscoveryKey");
            Assert.IsNull(control.PathWriterKey, "PathWriterKey");
        }

        /// <summary>
        /// Tests instantiating control with invalid enumeration value in the configuration file.
        /// Default value should then be used.
        /// </summary>
        [Test]
        public void TestWithConfigEnumIntInvalid()
        {
            TestCommon.LoadConfigFile(@"..\..\test_files\TestBctc\EnumIntInvalid.xml");
            control = new BreadCrumbTrailControl();

            // make sure all properties (including invalid values) are initialized to default
            Assert.AreEqual(":", control.PathSeparator, "PathSeparator");
            Assert.IsNull(control.PathSeparatorStyle, "PathSeparatorStyle");
            Assert.IsNull(control.NodeStyle, "NodeStyle");
            Assert.IsNull(control.RootNodeStyle, "RootNodeStyle");
            Assert.IsNull(control.CurrentNodeStyle, "CurrentNodeStyle");
            Assert.IsNull(control.MouseOverStyle, "MouseOverStyle");
            Assert.AreEqual(PathDirection.RootToCurrent, control.PathDirection, "PathDirection");
            Assert.AreEqual(ErrorDisposition.SilentlyIgnore, control.ErrorDisposition, "ErrorDisposition");
            Assert.IsNull(control.NodeDiscoveryKey, "NodeDiscoveryKey");
            Assert.IsNull(control.PathDiscoveryKey, "PathDiscoveryKey");
            Assert.IsNull(control.PathWriterKey, "PathWriterKey");
        }

        /// <summary>
        /// Tests instantiating control with empty enumeration value in the configuration file.
        /// Default value should then be used.
        /// </summary>
        [Test]
        public void TestWithConfigEnumEmpty()
        {
            TestCommon.LoadConfigFile(@"..\..\test_files\TestBctc\EnumEmpty.xml");
            control = new BreadCrumbTrailControl();

            // make sure all properties (including empty values) are initialized to default
            Assert.AreEqual(":", control.PathSeparator, "PathSeparator");
            Assert.IsNull(control.PathSeparatorStyle, "PathSeparatorStyle");
            Assert.IsNull(control.NodeStyle, "NodeStyle");
            Assert.IsNull(control.RootNodeStyle, "RootNodeStyle");
            Assert.IsNull(control.CurrentNodeStyle, "CurrentNodeStyle");
            Assert.IsNull(control.MouseOverStyle, "MouseOverStyle");
            Assert.AreEqual(PathDirection.RootToCurrent, control.PathDirection, "PathDirection");
            Assert.AreEqual(ErrorDisposition.SilentlyIgnore, control.ErrorDisposition, "ErrorDisposition");
            Assert.IsNull(control.NodeDiscoveryKey, "NodeDiscoveryKey");
            Assert.IsNull(control.PathDiscoveryKey, "PathDiscoveryKey");
            Assert.IsNull(control.PathWriterKey, "PathWriterKey");
        }
    }
}
